from tests.utils import service_ok

import pytest

from owslib.ogcapi.processes import Processes

SERVICE_URL = 'https://demo.pygeoapi.io/master'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason='service is unreachable')
def test_ogcapi_processes_pygeoapi():
    p = Processes(SERVICE_URL)

    assert p.url == 'https://demo.pygeoapi.io/master/'
    assert p.url_query_string is None

    api = p.api()
    assert api['components']['parameters'] is not None
    paths = api['paths']
    assert paths is not None
    assert paths['/processes/hello-world'] is not None

    conformance = p.conformance()
    assert len(conformance['conformsTo']) > 0

    collections = p.collections()
    assert len(collections) > 0

    processes = p.processes()
    assert len(processes) == 1

    hello_world = p.process('hello-world')
    assert hello_world['id'] == 'hello-world'
    assert hello_world['title'] == 'Hello World'



import pytest

import owslib
from owslib.etree import etree
from owslib.wfs import WebFeatureService
from owslib.wms import WebMapService
from tests.utils import service_ok


WMS_SERVICE_URL = 'https://www.dov.vlaanderen.be/geoserver/gw_meetnetten/' \
                  'wms?request=GetCapabilities'

WFS_SERVICE_URL = 'https://www.dov.vlaanderen.be/geoserver/gw_meetnetten/' \
                  'wfs?request=GetCapabilities'


@pytest.fixture
def mp_wfs_100_nometadata(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WFS
    version 1.0.0, not containing MetadataURLs.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wfs_dov_getcapabilities_100_nometadata.xml',
                  'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.feature.common.WFSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_wfs_110(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WFS
    version 1.1.0.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wfs_dov_getcapabilities_110.xml', 'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.feature.common.WFSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_wfs_110_nometadata(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WFS
    version 1.1.0, not containing MetadataURLs.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wfs_dov_getcapabilities_110_nometadata.xml',
                  'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.feature.common.WFSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_wfs_200(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WFS
    version 2.0.0.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wfs_dov_getcapabilities_200.xml',
                  'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.feature.common.WFSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_wfs_200_nometadata(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WFS
    version 2.0.0, not containing MetadataURLs.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wfs_dov_getcapabilities_200_nometadata.xml',
                  'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.feature.common.WFSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_wms_111_nometadata(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WMS
    version 1.1.1, not containing MetadataURLs.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wms_dov_getcapabilities_111_nometadata.xml',
                  'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.map.common.WMSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_wms_130(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WMS
    version 1.3.0.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wms_dov_getcapabilities_130.xml', 'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.map.common.WMSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_wms_130_nometadata(monkeypatch):
    """Monkeypatch the call to the remote GetCapabilities request of WMS
    version 1.3.0, not containing MetadataURLs.

    Parameters
    ----------
    monkeypatch : pytest.fixture
        PyTest monkeypatch fixture.

    """
    def read(*args, **kwargs):
        with open('tests/resources/wms_dov_getcapabilities_130_nometadata'
                  '.xml', 'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(
        owslib.map.common.WMSCapabilitiesReader, 'read', read)


@pytest.fixture
def mp_remote_md(monkeypatch):
    def openURL(*args, **kwargs):
        with open('tests/resources/csw_dov_getrecordbyid.xml', 'r') as f:
            data = f.read()
            if type(data) is not bytes:
                data = data.encode('utf-8')
            data = etree.fromstring(data)
        return data

    monkeypatch.setattr(owslib.util, 'openURL', openURL)


class TestOffline(object):
    def test_wfs_100_noremotemd_parse_all(self, mp_wfs_100_nometadata):
        """Test the remote metadata parsing for WFS 1.0.0.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_100_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.0.0',
                                parse_remote_metadata=True)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_100_noremotemd_parse_single(self, mp_wfs_100_nometadata):
        """Test the remote metadata parsing for WFS 1.0.0.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_100_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.0.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_100_noremotemd_parse_none(self, mp_wfs_100_nometadata):
        """Test the remote metadata parsing for WFS 1.0.0.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wfs_100_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.0.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_110_noremotemd_parse_all(self, mp_wfs_110_nometadata):
        """Test the remote metadata parsing for WFS 1.1.0.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_110_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.1.0',
                                parse_remote_metadata=True)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_110_noremotemd_parse_single(self, mp_wfs_110_nometadata):
        """Test the remote metadata parsing for WFS 1.1.0.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_110_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.1.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_110_noremotemd_parse_none(self, mp_wfs_110_nometadata):
        """Test the remote metadata parsing for WFS 1.1.0.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wfs_110_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.1.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_110_remotemd_parse_all(self, mp_wfs_110, mp_remote_md):
        """Test the remote metadata parsing for WFS 1.1.0.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns remote metadata
        if MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_110 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.
        mp_remote_md : pytest.fixture
            Monkeypatch the call to the remote metadata.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.1.0',
                                parse_remote_metadata=True)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 1

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    def test_wfs_110_remotemd_parse_single(self, mp_wfs_110, mp_remote_md):
        """Test the remote metadata parsing for WFS 1.1.0.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns remote metadata
        if MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_110 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.
        mp_remote_md : pytest.fixture
            Monkeypatch the call to the remote metadata.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.1.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 1

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    def test_wfs_110_remotemd_parse_none(self, mp_wfs_110):
        """Test the remote metadata parsing for WFS 1.1.0.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wfs_110 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='1.1.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_200_noremotemd_parse_all(self, mp_wfs_200_nometadata):
        """Test the remote metadata parsing for WFS 2.0.0.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_200_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='2.0.0',
                                parse_remote_metadata=True)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_200_noremotemd_parse_single(self, mp_wfs_200_nometadata):
        """Test the remote metadata parsing for WFS 2.0.0.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_200_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='2.0.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_200_noremotemd_parse_none(self, mp_wfs_200_nometadata):
        """Test the remote metadata parsing for WFS 2.0.0.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wfs_200_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='2.0.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wfs_200_remotemd_parse_all(self, mp_wfs_200, mp_remote_md):
        """Test the remote metadata parsing for WFS 2.0.0.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns remote metadata
        if MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_200 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.
        mp_remote_md : pytest.fixture
            Monkeypatch the call to the remote metadata.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='2.0.0',
                                parse_remote_metadata=True)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 1

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    def test_wfs_200_remotemd_parse_single(self, mp_wfs_200, mp_remote_md):
        """Test the remote metadata parsing for WFS 2.0.0.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns remote metadata
        if MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wfs_200 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.
        mp_remote_md : pytest.fixture
            Monkeypatch the call to the remote metadata.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='2.0.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 1

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    def test_wfs_200_remotemd_parse_none(self, mp_wfs_200):
        """Test the remote metadata parsing for WFS 2.0.0.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wfs_200 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wfs = WebFeatureService(url='http://localhost/not_applicable',
                                version='2.0.0',
                                parse_remote_metadata=False)
        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wms_111_noremotemd_parse_all(self, mp_wms_111_nometadata):
        """Test the remote metadata parsing for WMS 1.1.1.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wms_111_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.1.1',
                            parse_remote_metadata=True)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wms_111_noremotemd_parse_single(self, mp_wms_111_nometadata):
        """Test the remote metadata parsing for WMS 1.1.1.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wms_111_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.1.1',
                            parse_remote_metadata=False)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wms_111_noremotemd_parse_none(self, mp_wms_111_nometadata):
        """Test the remote metadata parsing for WMS 1.1.1.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wms_111_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.1.1',
                            parse_remote_metadata=False)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wms_130_remotemd_parse_all(self, mp_wms_130):
        """Test the remote metadata parsing for WMS 1.3.0.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns remote metadata
        if MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wms_130 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.3.0',
                            parse_remote_metadata=True)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 1

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    def test_wms_130_remotemd_parse_single(self, mp_wms_130):
        """Test the remote metadata parsing for WMS 1.3.0.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns remote metadata
        if MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wms_130 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.3.0',
                            parse_remote_metadata=False)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 1

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    def test_wms_130_remotemd_parse_none(self, mp_wms_130):
        """Test the remote metadata parsing for WMS 1.3.0.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wms_130 : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.3.0',
                            parse_remote_metadata=False)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wms_130_noremotemd_parse_all(self, mp_wms_130_nometadata):
        """Test the remote metadata parsing for WMS 1.3.0.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wms_130_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.3.0',
                            parse_remote_metadata=True)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wms_130_noremotemd_parse_single(self, mp_wms_130_nometadata):
        """Test the remote metadata parsing for WMS 1.3.0.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        Parameters
        ----------
        mp_wms_130_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.3.0',
                            parse_remote_metadata=False)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    def test_wms_130_noremotemd_parse_none(self, mp_wms_130_nometadata):
        """Test the remote metadata parsing for WMS 1.3.0.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        Parameters
        ----------
        mp_wms_130_nometadata : pytest.fixture
            Monkeypatch the call to the remote GetCapabilities request.

        """
        wms = WebMapService(url='http://localhost/not_applicable',
                            version='1.3.0',
                            parse_remote_metadata=False)
        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0


class TestOnline(object):
    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WFS_SERVICE_URL),
                        reason="WFS service is unreachable")
    @pytest.mark.parametrize("wfs_version", ["1.1.0", "2.0.0"])
    def test_wfs_remotemd_parse_single(self, wfs_version):
        """Test the remote metadata parsing for WFS.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns remote metadata.

        """
        wfs = WebFeatureService(url=WFS_SERVICE_URL,
                                version=wfs_version,
                                parse_remote_metadata=False)

        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) > 0

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WFS_SERVICE_URL),
                        reason="WFS service is unreachable")
    @pytest.mark.parametrize("wfs_version", ["1.1.0", "2.0.0"])
    def test_wfs_remotemd_parse_all(self, wfs_version):
        """Test the remote metadata parsing for WFS.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns remote metadata.

        """
        wfs = WebFeatureService(url=WFS_SERVICE_URL,
                                version=wfs_version,
                                parse_remote_metadata=True)

        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) > 0

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WFS_SERVICE_URL),
                        reason="WFS service is unreachable")
    @pytest.mark.parametrize("wfs_version", ["1.1.0", "2.0.0"])
    def test_wfs_remotemd_parse_none(self, wfs_version):
        """Test the remote metadata parsing for WFS.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        """
        wfs = WebFeatureService(url=WFS_SERVICE_URL,
                                version=wfs_version,
                                parse_remote_metadata=False)

        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WFS_SERVICE_URL),
                        reason="WFS service is unreachable")
    @pytest.mark.parametrize("wfs_version", ["1.0.0"])
    def test_wfs_noremotemd_parse_single(self, wfs_version):
        """Test the remote metadata parsing for WFS.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns no remote metadata
        if no MetadataURLs are available in the GetCapabilities.

        """
        wfs = WebFeatureService(url=WFS_SERVICE_URL,
                                version=wfs_version,
                                parse_remote_metadata=False)

        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WFS_SERVICE_URL),
                        reason="WFS service is unreachable")
    @pytest.mark.parametrize("wfs_version", ["1.0.0"])
    def test_wfs_noremotemd_parse_all(self, wfs_version):
        """Test the remote metadata parsing for WFS.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns no remote
        metadata if no MetadataURLs are available in the GetCapabilities.

        """
        wfs = WebFeatureService(url=WFS_SERVICE_URL,
                                version=wfs_version,
                                parse_remote_metadata=True)

        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WFS_SERVICE_URL),
                        reason="WFS service is unreachable")
    @pytest.mark.parametrize("wfs_version", ["1.0.0"])
    def test_wfs_noremotemd_parse_none(self, wfs_version):
        """Test the remote metadata parsing for WFS.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        """
        wfs = WebFeatureService(url=WFS_SERVICE_URL,
                                version=wfs_version,
                                parse_remote_metadata=False)

        assert 'gw_meetnetten:meetnetten' in wfs.contents
        layer = wfs.contents['gw_meetnetten:meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WMS_SERVICE_URL),
                        reason="WMS service is unreachable")
    @pytest.mark.parametrize("wms_version", ["1.3.0"])
    def test_wms_remotemd_parse_single(self, wms_version):
        """Test the remote metadata parsing for WMS.

        Tests parsing the remote metadata for a single layer.

        Test whether the method is available and returns remote metadata.

        """
        wms = WebMapService(url=WMS_SERVICE_URL,
                            version=wms_version,
                            parse_remote_metadata=False)

        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) > 0

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WMS_SERVICE_URL),
                        reason="WMS service is unreachable")
    @pytest.mark.parametrize("wms_version", ["1.3.0"])
    def test_wms_remotemd_parse_all(self, wms_version):
        """Test the remote metadata parsing for WMS.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns remote metadata.

        """
        wms = WebMapService(url=WMS_SERVICE_URL,
                            version=wms_version,
                            parse_remote_metadata=True)

        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) > 0

        for m in mdrecords:
            assert type(m) is owslib.iso.MD_Metadata

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WMS_SERVICE_URL),
                        reason="WMS service is unreachable")
    @pytest.mark.parametrize("wms_version", ["1.3.0"])
    def test_wms_remotemd_parse_none(self, wms_version):
        """Test the remote metadata parsing for WMS.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        """
        wms = WebMapService(url=WMS_SERVICE_URL,
                            version=wms_version,
                            parse_remote_metadata=False)

        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WMS_SERVICE_URL),
                        reason="WMS service is unreachable")
    @pytest.mark.parametrize("wms_version", ["1.1.1"])
    def test_wms_noremotemd_parse_single(self, wms_version):
        """Test the remote metadata parsing for WMS.

         Tests parsing the remote metadata for a single layer.

         Test whether the method is available and returns no remote metadata
         if no MetadataURLs are available in the GetCapabilities.

         """
        wms = WebMapService(url=WMS_SERVICE_URL,
                            version=wms_version,
                            parse_remote_metadata=False)

        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']
        layer.parse_remote_metadata()

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WMS_SERVICE_URL),
                        reason="WMS service is unreachable")
    @pytest.mark.parametrize("wms_version", ["1.1.1"])
    def test_wms_noremotemd_parse_all(self, wms_version):
        """Test the remote metadata parsing for WMS.

        Tests parsing the remote metadata for all layers.

        Test whether the method is available and returns no remote
        metadata if no MetadataURLs are available in the GetCapabilities.

        """
        wms = WebMapService(url=WMS_SERVICE_URL,
                            version=wms_version,
                            parse_remote_metadata=True)

        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0

    @pytest.mark.xfail
    @pytest.mark.online
    @pytest.mark.skipif(not service_ok(WMS_SERVICE_URL),
                        reason="WMS service is unreachable")
    @pytest.mark.parametrize("wms_version", ["1.1.1"])
    def test_wms_noremotemd_parse_none(self, wms_version):
        """Test the remote metadata parsing for WMS.

        Tests the case when no remote metadata is parsed.

        Test whether no remote metadata is returned.

        """
        wms = WebMapService(url=WMS_SERVICE_URL,
                            version=wms_version,
                            parse_remote_metadata=False)

        assert 'meetnetten' in wms.contents
        layer = wms.contents['meetnetten']

        mdrecords = layer.get_metadata()
        assert type(mdrecords) is list
        assert len(mdrecords) == 0



from tests.utils import service_ok

import pytest

from owslib.ogcapi.records import Records

SERVICE_URL = 'https://demo.pygeoapi.io/master/'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason='service is unreachable')
def test_ogcapi_records_pygeoapi():
    w = Records(SERVICE_URL)

    assert w.url == SERVICE_URL
    assert w.url_query_string is None

    api = w.api()
    assert api['components']['parameters'] is not None
    paths = api['paths']
    assert paths is not None
    assert paths['/collections/dutch-metadata'] is not None

    conformance = w.conformance()
    assert len(conformance['conformsTo']) > 8

    collections = w.collections()
    assert len(collections) > 0

    records = w.records()
    assert len(records) == 1

    dutch_metacat = w.collection('dutch-metadata')
    assert dutch_metacat['id'] == 'dutch-metadata'
    assert dutch_metacat['title'] == 'Sample metadata records from Dutch Nationaal georegister'  # noqa
    assert dutch_metacat['description'] == 'Sample metadata records from Dutch Nationaal georegister'  # noqa
    assert w.request == f'{SERVICE_URL}collections/dutch-metadata'
    assert w.response is not None
    assert isinstance(w.response, dict)

    dutch_metacat_queryables = w.collection_queryables('dutch-metadata')
    assert len(dutch_metacat_queryables['properties']) == 10

    # Minimum of limit param is 1
    with pytest.raises(RuntimeError):
        dutch_metacat_query = w.collection_items('dutch-metadata', limit=0)

    dutch_metacat_query = w.collection_items('dutch-metadata', limit=1)
    assert dutch_metacat_query['numberMatched'] == 308
    assert dutch_metacat_query['numberReturned'] == 1
    assert len(dutch_metacat_query['features']) == 1

    dutch_metacat_query = w.collection_items('dutch-metadata', q='Wegpanorama')
    assert dutch_metacat_query['numberMatched'] == 3
    assert dutch_metacat_query['numberReturned'] == 3
    assert len(dutch_metacat_query['features']) == 3



# Simulate a WPS GetCapabilities invocation.
# This test does not execute any live HTTP request, rather it parses XML files containing pre-made HTTP responses.

from tests.utils import resource_file
from owslib.wps import WebProcessingService


def test_wps_getcapabilities_usgs():
    # Initialize WPS client
    wps = WebProcessingService('http://cida.usgs.gov/gdp/process/WebProcessingService', skip_caps=True)

    # Execute fake invocation of GetCapabilities operation by parsing cached response from USGS service
    xml = open(resource_file('wps_USGSCapabilities.xml'), 'rb').read()
    wps.getcapabilities(xml=xml)

    # Check WPS description
    assert wps.updateSequence is not None
    assert wps.identification.type == 'WPS'
    assert wps.identification.title == 'Geo Data Portal WPS Implementation'
    assert wps.identification.abstract == 'A Geo Data Portal Service based on the 52north implementation of WPS 1.0.0'

    # Check available operations
    operations = [op.name for op in wps.operations]
    assert operations == [
        'GetCapabilities',
        'DescribeProcess',
        'Execute']

    # Check high level process descriptions
    processes = [(p.identifier, p.title) for p in wps.processes]
    assert processes == [
        ('gov.usgs.cida.gdp.wps.algorithm.filemanagement.ReceiveFiles', 'gov.usgs.cida.gdp.wps.algorithm.filemanagement.ReceiveFiles'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.discovery.CalculateWCSCoverageInfo', 'gov.usgs.cida.gdp.wps.algorithm.discovery.CalculateWCSCoverageInfo'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.communication.EmailWhenFinishedAlgorithm', 'gov.usgs.cida.gdp.wps.algorithm.communication.EmailWhenFinishedAlgorithm'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.communication.GeoserverManagementAlgorithm', 'gov.usgs.cida.gdp.wps.algorithm.communication.GeoserverManagementAlgorithm'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.discovery.GetWcsCoverages', 'gov.usgs.cida.gdp.wps.algorithm.discovery.GetWcsCoverages'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.filemanagement.GetWatersGeom', 'gov.usgs.cida.gdp.wps.algorithm.filemanagement.GetWatersGeom'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.discovery.ListOpendapGrids', 'gov.usgs.cida.gdp.wps.algorithm.discovery.ListOpendapGrids'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.filemanagement.CreateNewShapefileDataStore', 'gov.usgs.cida.gdp.wps.algorithm.filemanagement.CreateNewShapefileDataStore'),  # noqa
        ('gov.usgs.cida.gdp.wps.algorithm.discovery.GetGridTimeRange', 'gov.usgs.cida.gdp.wps.algorithm.discovery.GetGridTimeRange'),  # noqa
        ]



import pytest
from owslib.wfs import WebFeatureService
from owslib import fes2
from owslib.gml import Point
from owslib.namespaces import Namespaces
from owslib import util
import json

n = Namespaces()
FES_NAMESPACE = n.get_namespace("fes")
GML32_NAMESPACE = n.get_namespace("gml32")


SERVICE_URL = "http://soggy.zoology.ubc.ca:8080/geoserver/wfs"


def test_raw_filter():
    """Just inspect the filter object (not embedded in a getfeature request)."""
    point = Point(id="qc", srsName="http://www.opengis.net/gml/srs/epsg.xml#4326", pos=[-71, 46])
    f = fes2.Filter(
        fes2.And([fes2.Intersects(propertyname="the_geom", geometry=point),
                  fes2.PropertyIsLike("name", "value")]
                 )
    )

    xml = f.toXML()

    # Fairly basic test
    xml.find(util.nspath("Filter", FES_NAMESPACE))
    xml.find(util.nspath("And", FES_NAMESPACE))
    xml.find(util.nspath("Intersects", FES_NAMESPACE))
    xml.find(util.nspath("Point", GML32_NAMESPACE))

@pytest.mark.xfail
@pytest.mark.online
def test_filter():
    """A request without filtering will yield 600 entries. With filtering we expect only one.

    Note that this type of topological filtering only works (so far) with WFS 2.0.0 and POST requests.
    """
    wfs = WebFeatureService(SERVICE_URL, version="2.0.0")
    layer = "psf:level4"
    point = Point(id="random", srsName="http://www.opengis.net/gml/srs/epsg.xml#4326", pos=[-129.8, 55.44])
    f = fes2.Filter(fes2.Contains(propertyname="geom", geometry=point))
    r = wfs.getfeature(layer, outputFormat="application/json", method="POST", filter=f.toXML())
    assert json.load(r)["totalFeatures"] == 1



import logging
import os
import sys
import requests
from owslib.etree import etree, ElementType
from urllib.parse import urlparse


def setup_logging(loglevel='INFO'):
    """Helper function to setup logging for tests"""
    logger = logging.getLogger('owslib')
    logger.setLevel(getattr(logging, loglevel))
    sh = logging.StreamHandler(sys.stdout)
    formatter = logging.Formatter('%(message)s')
    sh.setFormatter(formatter)
    logger.addHandler(sh)
    return logger


def resource_file(filepath):
    return os.path.join(test_directory(), 'resources', filepath)


def test_directory():
    """Helper function to return path to the tests directory"""
    return os.path.dirname(__file__)


def scratch_directory():
    """Helper function to return path to the tests scratch directory"""
    return os.path.join(test_directory(), 'scratch')


def scratch_file(filename):
    """Helper function to return file path in the tests scratch directory"""
    return os.path.join(scratch_directory(), filename)


def compare_xml(a, b):
    if not isinstance(a, ElementType):
        a = etree.fromstring(a)
    if not isinstance(b, ElementType):
        b = etree.fromstring(b)

    return compare_elements(a, b)


def compare_elements(a, b):
    # Tag
    if a.tag != b.tag:
        return False
    # Value
    if a.text != b.text:
        return False
    # Attributes
    if sorted(a.items()) != sorted(b.items()):
        return False
    # Children
    if len(list(a)) != len(list(b)):
        return False
    # Recurse
    for ac, bc in zip(list(a), list(b)):
        if not compare_elements(ac, bc):
            return False

    return True


def cast_tuple_int_list(tup):
    """Set tuple float values to int for more predictable test results"""
    return [int(a) for a in tup]


def cast_tuple_int_list_srs(tup):
    tup2 = cast_tuple_int_list(tup[:4])
    tup2.append(tup[-1])
    return tup2


def sorted_url_query(url):
    return sorted(urlparse(url).query.split("&"))


def service_ok(url, timeout=5):
    try:
        resp = requests.head(url, allow_redirects=True, timeout=timeout)
        if 'html' in resp.headers['content-type']:
            ok = False
        else:
            ok = resp.ok
    except requests.exceptions.ReadTimeout:
        ok = False
    except requests.exceptions.ConnectTimeout:
        ok = False
    except Exception:
        ok = False
    return ok



from tests.utils import service_ok

import pytest

from owslib.ogcapi.features import Features

SERVICE_URL = 'https://www.ldproxy.nrw.de/rest/services/kataster/?f=json'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason='service is unreachable')
def test_ogcapi_features_ldproxy():
    w = Features(SERVICE_URL)

    assert w.url == 'https://www.ldproxy.nrw.de/rest/services/kataster/'
    assert w.url_query_string == 'f=json'

    conformance = w.conformance()
    assert len(conformance['conformsTo']) == 5

    feature_collections = w.feature_collections()
    assert len(feature_collections) >= 0

    # TODO: remove pytest.raises once ldproxy is fixed/updated
    with pytest.raises(RuntimeError):
        api = w.api()
        assert api['components']['parameters'] is not None
        assert api['paths'] is not None



# Example script that performs a set of (small) live requests versus the live CEDA WPS service

from owslib.wps import WebProcessingService, WPSExecution, WFSFeatureCollection, WFSQuery, GMLMultiPolygonFeatureCollection, monitorExecution, ComplexData, printInputOutput
from owslib.util import dump

verbose = True
wps = WebProcessingService('http://ceda-wps2.badc.rl.ac.uk/wps', verbose=verbose)

# 1) GetCapabilities
# GET request: http://ceda-wps2.badc.rl.ac.uk/wps?Service=WPS&Request=GetCapabilities&Format=text/xml
wps.getcapabilities()

print('WPS Identification type: %s' % wps.identification.type)
print('WPS Identification title: %s' % wps.identification.title)
print('WPS Identification abstract: %s' % wps.identification.abstract)
for operation in wps.operations:
    print('WPS Operation: %s' % operation.name)
for process in wps.processes:
    print('WPS Process: identifier=%s title=%s' % (process.identifier, process.title))
    
# 2) DescribeProcess
# GET request: http://ceda-wps2.badc.rl.ac.uk/wps?identifier=DoubleIt&version=1.0.0&request=DescribeProcess&service=WPS
process = wps.describeprocess('DoubleIt')
print('WPS Process: identifier=%s' % process.identifier)
print('WPS Process: title=%s' % process.title)
print('WPS Process: abstract=%s' % process.abstract)
for input in process.dataInputs:
    print('Process input:')
    printInputOutput(input, indent='\t')
for output in process.processOutputs:
    print('Process output:')
    printInputOutput(output, indent='\t')

# 3) Execute
# POST request:
# Note: not working, requires openid login ?
#processid = "DoubleIt"
#inputs = [ ("NumberToDouble","1") ]
#output = "OutputXML"
#execution = wps.execute(processid, inputs, output)

#monitorExecution(execution)



from unittest import mock

import pytest
from tests.utils import service_ok

from owslib.wms import WebMapService
from owslib.util import ServiceException
from owslib.util import ResponseWrapper


SERVICE_URL = 'http://mesonet.agron.iastate.edu/cgi-bin/wms/nexrad/n0r-t.cgi'
NCWMS2_URL = "http://wms.stccmop.org:8080/ncWMS2/wms"


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMS service is unreachable")
def test_wms_getmap_111():
    """MESONET GetMap 1.1.1"""
    wms = WebMapService(SERVICE_URL, version='1.1.1')
    assert wms.request == '{}?service=WMS&request=GetCapabilities&version=1.1.1'.format(SERVICE_URL)
    rsp = wms.getmap(
        layers=['nexrad_base_reflect'],
        styles=['default'],
        srs='EPSG:4326',
        bbox=(-126, 24, -66, 50),
        size=(250, 250),
        format='image/jpeg',
        transparent=True)
    import owslib.util
    assert type(rsp) is ResponseWrapper


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMS service is unreachable")
def test_wms_getmap_111_service_exception():
    """GetMap 1.1.1 ServiceException for an invalid CRS"""
    wms = WebMapService(SERVICE_URL, version='1.1.1')
    try:
        wms.getmap(
            layers=['nexrad_base_reflect'],
            styles=['default'],
            srs='EPSG:4328',
            bbox=(-126, 24, -66, 50),
            size=(250, 250),
            format='image/jpeg',
            transparent=True)
    except ServiceException as e:
        assert "msWMSLoadGetMapParams(): WMS server error. Invalid SRS given : SRS must be valid for all requested layers." in str(e)  # noqa
    else:
        assert False


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMS service is unreachable")
def test_wms_getmap_130():
    """GetMap 1.3.0"""
    wms = WebMapService(SERVICE_URL, version='1.3.0')
    rsp = wms.getmap(
        layers=['nexrad_base_reflect'],
        styles=['default'],
        srs='EPSG:4326',
        bbox=(-126, 24, -66, 50),
        size=(250, 250),
        format='image/jpeg',
        transparent=True)
    assert type(rsp) is ResponseWrapper


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMS service is unreachable")
def test_wms_getmap_130_service_exception():
    """GetMap 1.3.0 ServiceException for an invalid CRS"""
    wms = WebMapService(SERVICE_URL, version='1.3.0')
    try:
        wms.getmap(
            layers=['nexrad_base_reflect'],
            styles=['default'],
            srs='EPSG:4328',
            bbox=(-126, 24, -66, 50),
            size=(250, 250),
            format='image/jpeg',
            transparent=True)
    except ServiceException as e:
        assert "msWMSLoadGetMapParams(): WMS server error. Invalid CRS given : CRS must be valid for all requested layers." in str(e)  # noqa
    else:
        assert False


SERVICE_URL_NATIONAL_MAP = 'http://services.nationalmap.gov/ArcGIS/services/geonames/MapServer/WMSServer'


@pytest.mark.online
@pytest.mark.skip(reason="this is a flaky test")
# @pytest.mark.skipif(not service_ok(SERVICE_URL_NATIONAL_MAP),
#                    reason="WMS service is unreachable")
def test_getmap_130_national_map():
    """National Map"""
    # TODO: use flaky tests or fix it: https://pypi.python.org/pypi/pytest-ignore-flaky
    url = SERVICE_URL_NATIONAL_MAP
    wms = WebMapService(url, version='1.3.0')
    rsp = wms.getmap(
        layers=['3'],
        styles=['default'],
        srs='CRS:84',
        bbox=(-176.646, 17.7016, -64.8017, 71.2854),
        size=(500, 300),
        format='image/png',
        transparent=True)
    assert type(rsp) is ResponseWrapper
    assert "service=WMS" in wms.request
    assert "version=1.3.0" in wms.request
    assert "request=GetMap" in wms.request
    assert "layers=3" in wms.request
    assert "styles=default" in wms.request
    assert "crs=CRS%3A84" in wms.request
    assert "box=-176.646%2C17.7016%2C-64.8017%2C71.2854" in wms.request
    assert "width=500" in wms.request
    assert "height=300" in wms.request
    assert "format=image%2Fpng" in wms.request
    assert "transparent=TRUE" in wms.request


@pytest.mark.online
@pytest.mark.skipif(not service_ok(NCWMS2_URL), reason="WMS service is unreachable")
def test_ncwms2():
    """Test with an ncWMS2 server.
    """
    # Note that this does not exercise the bug in https://github.com/geopython/OWSLib/issues/556
    wms = WebMapService(NCWMS2_URL, version='1.3.0')
    rsp = wms.getmap(
        layers=['f33_thredds/min_temp'],
        styles=['default'],
        srs='CRS:84',
        bbox=(-124.17, 46.02, -123.29, 46.38),
        size=(256, 256),
        format='image/png',
        transparent=True,
        mode='32bit',

    )
    assert type(rsp) is ResponseWrapper
    assert "service=WMS" in wms.request
    assert "version=1.3.0" in wms.request
    assert "request=GetMap" in wms.request
    assert "layers=f33_thredds/min_temp" in wms.request
    assert "styles=default" in wms.request
    assert "crs=CRS%3A84" in wms.request
    assert "width=256" in wms.request
    assert "height=256" in wms.request
    assert "format=image%2Fpng" in wms.request
    assert "transparent=TRUE" in wms.request


@pytest.mark.parametrize('wms_version', ['1.1.1', '1.3.0'])
def test_wms_sends_headers(wms_version):
    """Test that if headers are provided in the WMS class they are sent
    when performing HTTP requests (in this case for GetCapabilities)
    """

    with mock.patch('owslib.util.requests.request', side_effect=RuntimeError) as mock_request:
        try:
            WebMapService(
                'http://example.com/wms',
                version=wms_version,
                headers={'User-agent': 'my-app/1.0'}
            )
        except RuntimeError:

            assert mock_request.called
            assert mock_request.call_args[1]['headers'] == {'User-agent': 'my-app/1.0'}



from unittest import mock
from tests.utils import service_ok

import pytest

from owslib.csw import CatalogueServiceWeb


def test_csw_sends_headers():
    """
    Test that if headers are provided in the CSW class they are sent
    when performing HTTP requests (in this case for GetCapabilities)
    """

    with mock.patch('owslib.util.requests.request', side_effect=RuntimeError) as mock_request:
        try:
            CatalogueServiceWeb(
                'http://example.com/csw',
                version='2.0.2',
                headers={'User-agent': 'my-app/1.0'}
            )
        except RuntimeError:
            assert mock_request.called
            assert mock_request.call_args[1]['headers'] == {'User-agent': 'my-app/1.0'}

    with mock.patch('owslib.util.requests.request', side_effect=RuntimeError) as mock_request:
        try:
            CatalogueServiceWeb(
                'http://example.com/csw',
                version='3.0.0',
                headers={'User-agent': 'my-app/1.0'}
            )
        except RuntimeError:
            assert mock_request.called
            assert mock_request.call_args[1]['headers'] == {'Accept': 'application/xml', 'User-agent': 'my-app/1.0'}



from owslib.feature import postrequest
from owslib.etree import etree
from owslib.namespaces import Namespaces
from owslib import util
import pytest

n = Namespaces()
FES_NAMESPACE = n.get_namespace("fes")
GML_NAMESPACE = n.get_namespace("gml")
GML32_NAMESPACE = n.get_namespace("gml32")
OGC_NAMESPACE = n.get_namespace("ogc")
WFS_NAMESPACE = n.get_namespace("wfs")
WFS20_NAMESPACE = n.get_namespace("wfs20")

xml_filter_1_1_0 = """<?xml version="1.0" ?>
    <wfs:GetFeature service="WFS" version="1.1.0"
        xmlns:wfs="http://www.opengis.net/wfs"
        xmlns:ogc="http://www.opengis.net/ogc"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:schemaLocation="http://www.opengis.net/wfs
                            http://schemas.opengis.net/wfs/1.1.0/wfs.xsd">
        <ogc:Filter>
            <ogc:PropertyIsEqualTo>
                <ogc:PropertyName>status</ogc:PropertyName>
                <ogc:Literal>rejected</ogc:Literal>
            </ogc:PropertyIsEqualTo>
        </ogc:Filter>
    </wfs:GetFeature>
    """

xml_filter_2_0_0 = """<?xml version="1.0" ?>
    <wfs:GetFeature service="WFS" version="2.0.0"
        xmlns:wfs="http://www.opengis.net/wfs/2.0"
            xmlns:fes="http://www.opengis.net/fes/2.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://www.opengis.net/wfs/2.0
                                http://schemas.opengis.net/wfs/2.0.02.0/wfs.xsd">
        <fes:Filter>
            <fes:PropertyIsEqualTo>
                <fes:ValueReference>status</fes:ValueReference>
                <fes:Literal>rejected</fes:Literal>
            </fes:PropertyIsEqualTo>
        </fes:Filter>
    </wfs:GetFeature>
    """

typename = "ns:typename"


@pytest.fixture
def requestv110():
    request = postrequest.PostRequest_1_1_0()
    request.create_query(typename)
    return request


@pytest.fixture
def requestv200():
    request = postrequest.PostRequest_2_0_0()
    request.create_query(typename)
    return request


class TestPostRequest_v_1_1_0():

    def test_minimal_query(self, requestv110):

        elem = requestv110._root.find(util.nspath("Query", WFS_NAMESPACE))

        assert elem.get("typeName") == typename

    def test_basic_query(self, requestv110):
        requestv110.set_maxfeatures(2)
        requestv110.set_outputformat('JSON')
        requestv110.set_startindex(0)
        requestv110.set_propertyname(["genericName"])

        root = requestv110._root
        elem = requestv110._query.findtext("PropertyName")

        assert root.get("maxFeatures") == '2'
        assert root.get("outputFormat") == 'JSON'
        assert root.get("startIndex") == '0'
        assert elem == "genericName"

    def test_featureid_query_single(self, requestv110):
        requestv110.set_featureid(["1"])

        filter_elem = requestv110._query.find(util.nspath("Filter", OGC_NAMESPACE))
        resource_elem = filter_elem.find(util.nspath("GmlObjectId", OGC_NAMESPACE))

        assert filter_elem is not None
        assert resource_elem.get(util.nspath('id', GML_NAMESPACE)) == '1'

    def test_featureid_query_multiple(self, requestv110):
        requestv110.set_featureid(["1", "2", "3"])

        filter_elem = requestv110._query.find(util.nspath("Filter", OGC_NAMESPACE))
        resource_elem = filter_elem.findall(util.nspath("GmlObjectId", OGC_NAMESPACE))
        ids = []
        for elem in resource_elem:
            ids.append(elem.get(util.nspath('id', GML_NAMESPACE)))

        assert filter_elem is not None
        assert len(ids) == 3
        assert ids[0] == "1"
        assert ids[1] == "2"
        assert ids[2] == "3"

    def test_sortby_query_single(self, requestv110):
        requestv110.set_sortby(["id"])

        sort_elem = requestv110._query.find(util.nspath("SortBy", OGC_NAMESPACE))
        sortprop_elem = sort_elem.find(util.nspath("SortProperty", OGC_NAMESPACE))
        propertyname = sortprop_elem.findtext(util.nspath("PropertyName", OGC_NAMESPACE))

        assert sort_elem is not None
        assert sortprop_elem is not None
        assert propertyname == "id"

    def test_sortby_query_multiple(self, requestv110):
        requestv110.set_sortby(["id", "type"])

        sort_elem = requestv110._query.find(util.nspath("SortBy", OGC_NAMESPACE))
        sortprop_elem = sort_elem.findall(util.nspath("SortProperty", OGC_NAMESPACE))
        propertyname = []
        for elem in sortprop_elem:
            propertyname.append(elem.findtext(util.nspath("PropertyName", OGC_NAMESPACE)))

        assert sort_elem is not None
        assert len(sortprop_elem) == 2
        assert propertyname[0] == "id"
        assert propertyname[1] == "type"

    def test_bbox_query(self, requestv110):
        requestv110.set_bbox([10, 10, 20, 20])

        filter_elem = requestv110._query.find(util.nspath("Filter", OGC_NAMESPACE))
        bbox_elem = filter_elem.find(util.nspath("BBOX", OGC_NAMESPACE))
        envel_elem = bbox_elem.find(util.nspath("Envelope", GML_NAMESPACE))
        lower_elem = envel_elem.findtext(util.nspath("lowerCorner", GML_NAMESPACE))
        higher_elem = envel_elem.findtext(util.nspath("upperCorner", GML_NAMESPACE))

        assert filter_elem is not None
        assert bbox_elem is not None
        assert envel_elem is not None
        assert lower_elem == "10 10"
        assert higher_elem == "20 20"

    def test_filter_query(self, requestv110):
        requestv110.set_filter(xml_filter_1_1_0)

        filter_elem = requestv110._query.find(util.nspath("Filter", OGC_NAMESPACE))
        equal_elem = filter_elem.find(util.nspath("PropertyIsEqualTo", OGC_NAMESPACE))
        propertyname = equal_elem.findtext(util.nspath("PropertyName", OGC_NAMESPACE))
        literal = equal_elem.findtext(util.nspath("Literal", OGC_NAMESPACE))

        assert filter_elem is not None
        assert equal_elem is not None
        assert propertyname == "status"
        assert literal == "rejected"


class TestPostRequest_v_2_0_0():

    def test_minimal_query(self, requestv200):
        elem = requestv200._root.find(util.nspath("Query", WFS20_NAMESPACE))

        assert elem.get("typenames") == typename

    def test_basic_query(self, requestv200):
        requestv200.set_maxfeatures(2)
        requestv200.set_outputformat('JSON')
        requestv200.set_startindex(0)
        requestv200.set_propertyname(["genericName"])

        root = requestv200._root
        elem = requestv200._query.findtext("PropertyName")

        assert root.get("count") == '2'
        assert root.get("outputformat") == 'JSON'
        assert root.get("startIndex") == '0'
        assert elem == "genericName"

    def test_create_storedquery(self):
        stored_params = {"city": "Washington", "elevation_m": "125"}
        request = postrequest.PostRequest_2_0_0()
        request.create_storedquery("stored_id_1", stored_params)

        stored_query_elem = request._root.find(util.nspath("StoredQuery", WFS20_NAMESPACE))
        assert stored_query_elem.get("id") == "stored_id_1"

        params = stored_query_elem.findall(util.nspath("Parameter", WFS20_NAMESPACE))
        assert params[0].get("name") == "city"
        assert params[0].text == "Washington"
        assert params[1].get("name") == "elevation_m"
        assert params[1].text == "125"

    def test_featureid_query_single(self, requestv200):
        requestv200.set_featureid(["1"])

        filter_elem = requestv200._query.find(util.nspath("Filter", FES_NAMESPACE))
        resource_elem = filter_elem.find(util.nspath("ResourceId", FES_NAMESPACE))

        assert filter_elem is not None
        assert resource_elem.get("rid") == "1"

    def test_featureid_query_multiple(self, requestv200):
        requestv200.set_featureid(["1", "2", "3"])

        filter_elem = requestv200._query.find(util.nspath("Filter", FES_NAMESPACE))
        resource_elems = filter_elem.findall(util.nspath("ResourceId", FES_NAMESPACE))
        ids = []
        for elem in resource_elems:
            ids.append(elem.get("rid"))

        assert filter_elem is not None
        assert len(ids) == 3
        assert ids[0] == "1"
        assert ids[1] == "2"
        assert ids[2] == "3"

    def test_sortby_query_single(self, requestv200):
        requestv200.set_sortby(["id"])

        sort_elem = requestv200._query.find(util.nspath("SortBy", FES_NAMESPACE))
        sortprop_elem = sort_elem.find(util.nspath("SortProperty", FES_NAMESPACE))
        propertyname = sortprop_elem.findtext(util.nspath("ValueReference", FES_NAMESPACE))

        assert sort_elem is not None
        assert sortprop_elem is not None
        assert propertyname == "id"

    def test_sortby_query_multiple(self, requestv200):
        requestv200.set_sortby(["id", "type"])

        sort_elem = requestv200._query.find(util.nspath("SortBy", FES_NAMESPACE))
        sortprop_elem = sort_elem.findall(util.nspath("SortProperty", FES_NAMESPACE))
        propertyname = []
        for elem in sortprop_elem:
            propertyname.append(elem.findtext(util.nspath("ValueReference", FES_NAMESPACE)))

        assert sort_elem is not None
        assert len(sortprop_elem) == 2
        assert propertyname[0] == "id"
        assert propertyname[1] == "type"

    def test_bbox_query(self, requestv200):
        requestv200.set_bbox([10, 10, 20, 20])

        filter_elem = requestv200._query.find(util.nspath("Filter", FES_NAMESPACE))
        bbox_elem = filter_elem.find(util.nspath("BBOX", FES_NAMESPACE))
        envel_elem = bbox_elem.find(util.nspath("Envelope", GML32_NAMESPACE))
        lower_elem = envel_elem.findtext(util.nspath("lowerCorner", GML32_NAMESPACE))
        higher_elem = envel_elem.findtext(util.nspath("upperCorner", GML32_NAMESPACE))

        assert filter_elem is not None
        assert bbox_elem is not None
        assert envel_elem is not None
        assert lower_elem == "10 10"
        assert higher_elem == "20 20"

    def test_filter_query(self, requestv200):
        requestv200.set_filter(xml_filter_2_0_0)

        filter_elem = requestv200._query.find(util.nspath("Filter", FES_NAMESPACE))
        equal_elem = filter_elem.find(util.nspath("PropertyIsEqualTo", FES_NAMESPACE))
        propertyname = equal_elem.findtext(util.nspath("ValueReference", FES_NAMESPACE))
        literal = equal_elem.findtext(util.nspath("Literal", FES_NAMESPACE))

        assert filter_elem is not None
        assert equal_elem is not None
        assert propertyname == "status"
        assert literal == "rejected"



import pytest


from tests.utils import resource_file
from owslib.wps import WebProcessingService, WPSExecution, Process, is_reference, Input
from owslib.etree import etree


@pytest.fixture
def wps():
    '''Returns a WPS instance'''
    # Initialize WPS client
    wps = WebProcessingService('http://example.org/wps', skip_caps=True)
    xml = open(resource_file('wps_CEDACapabilities.xml'), 'rb').read()
    wps.getcapabilities(xml=xml)
    return wps


def test_wps_getOperationByName(wps):
    wps.getOperationByName("GetCapabilities")
    wps.getOperationByName("DescribeProcess")
    wps.getOperationByName("Execute")
    # Unavailable operations
    with pytest.raises(KeyError):
        wps.getOperationByName("GetStatus")


def test_wps_checkStatus():
    execution = WPSExecution()
    xml = open(resource_file('wps_PMLExecuteResponse5.xml'), 'rb').read()
    execution.checkStatus(response=xml)
    assert execution.isSucceded()
    assert execution.creationTime == '2011-11-08T21:36:55Z'


def test_wps_process_representation(wps):
    p = wps.processes[0]
    assert repr(p) == '<owslib.wps.Process CDMSSubsetVariable>'
    assert str(p) == 'WPS Process: CDMSSubsetVariable, title=Writes a text file and returns an output.'


def test_wps_process_properties(wps):
    p = wps.processes[0]
    assert p.statusSupported is None
    assert p.storeSupported is None


def test_wps_process_with_invalid_identifier():
    p = Process(etree.Element('invalid'))
    assert repr(p) == '<owslib.wps.Process >'
    assert str(p) == 'WPS Process: , title='


def test_wps_literal_data_input_parsing_references():
    xml = open(resource_file('wps_inout_parsing.xml'), 'r').read()
    inputs = etree.fromstring(xml)
    for i, i_elem in enumerate(inputs):
        wps_in = Input(i_elem)
        assert wps_in.identifier == 'input{}'.format(i + 1)
        assert wps_in.dataType == 'string'


def test_wps_response_with_lineage():
    execution = WPSExecution()
    xml = open(resource_file('wps_HummingbirdExecuteResponse1.xml'), 'rb').read()
    execution.checkStatus(response=xml)
    assert execution.isSucceded()
    assert execution.creationTime == '2018-05-08T14:00:54Z'
    # check lineage input with literal data
    inp = execution.dataInputs[0]
    assert inp.identifier == 'test'
    assert inp.title == 'Select the test you want to run.'
    assert inp.abstract == 'CF-1.6=Climate and Forecast Conventions (CF)'
    assert inp.data[0] == 'CF-1.6'
    # check lineage input with reference
    inp = execution.dataInputs[1]
    assert inp.identifier == 'dataset'
    assert inp.title == 'Upload your NetCDF file here'
    assert inp.abstract == 'or enter a URL pointing to a NetCDF file.'
    assert inp.reference.startswith('https://www.esrl.noaa.gov/')
    # check output with reference
    outp = execution.processOutputs[0]
    assert outp.identifier == 'output'
    assert outp.title == 'Test Report'
    assert outp.abstract == 'Compliance checker test report.'
    assert outp.reference.startswith('http://localhost:8090/wpsoutputs')


def test_is_reference():
    assert is_reference('http://testing.org')
    assert is_reference(b'http://testing.org')
    assert not is_reference('mumbo jumbo')
    assert not is_reference(b'mumbo jumbo')



from tests.utils import cast_tuple_int_list, scratch_file
from tests.utils import service_ok
from owslib.wcs import WebCoverageService

import pytest

SERVICE_URL = 'http://www.idee.es/wcs/IDEE-WCS-UTM30N/wcsServlet'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WCS service is unreachable")
def test_wcs_idee():
    """
    COWS Web Coverage Service
    WCS Version 1.0.0

    rewritten doctest/wcs_idee.txt
    """
    wcs = WebCoverageService(SERVICE_URL)
    assert wcs.version == '1.0.0'
    assert wcs.url == SERVICE_URL
    assert wcs.identification.title == 'WCS UTM30N - MDT Peninsula y Baleares'
    assert wcs.identification.service == 'IDEE-WCS-UTM30N'
    assert wcs.provider.name == u'Instituto Geogr\xe1fico Nacional'
    assert sorted(wcs.contents.keys()) == [
        'MDT1000_peninsula_baleares',
        'MDT1000_peninsula_baleares_aspecto',
        'MDT1000_peninsula_baleares_pendientes',
        'MDT25_peninsula_ZIP',
        'MDT25_peninsula_aspecto',
        'MDT25_peninsula_pendientes',
        'MDT500_peninsula_baleares',
        'MDT500_peninsula_baleares_aspecto',
        'MDT500_peninsula_baleares_pendientes',
        'MDT_peninsula_baleares',
        'MDT_peninsula_baleares_aspecto',
        'MDT_peninsula_baleares_pendientes']
    cvg = wcs['MDT25_peninsula_pendientes']
    assert cvg.title == 'MDT25 Pendientes Peninsula'
    assert cast_tuple_int_list(cvg.boundingBoxWGS84) == [-8, 35, 3, 43]
    assert cvg.timelimits == []
    assert sorted(cvg.supportedFormats) == ['AsciiGrid', 'FloatGrid_Zip', 'GeoTIFF']
    assert sorted(map(lambda x: x.getcode(), cvg.supportedCRS)) == [
        'EPSG:23028',
        'EPSG:23029',
        'EPSG:23030',
        'EPSG:23030',
        'EPSG:23031',
        'EPSG:4230',
        'EPSG:4326']
    output = wcs.getCoverage(
        identifier='MDT25_peninsula_pendientes',
        bbox=(600000, 4200000, 601000, 4201000),
        crs='EPSG:23030', format='AsciiGrid', resX=25, resY=25)
    f = open(scratch_file('test_idee.grd'), 'wb')
    bytes_written = f.write(output.read())
    f.close()



from tests.utils import service_ok

import pytest

from owslib.csw import CatalogueServiceWeb as cs

SERVICE_URL = 'http://demo.pycsw.org/cite/csw'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason='service is unreachable')
def test_csw_pycsw():
    c = cs(SERVICE_URL, version='3.0.0')
    assert c.updateSequence is not None
    assert c.version == '3.0.0'
    assert c.identification.title == 'pycsw OGC CITE demo and Reference Implementation'
    assert c.constraints['MaxRecordDefault'].values[0] == '10'
    c.getrecords()
    assert c.results.get('returned') > 0
    assert c.results.get('nextrecord') > 0
    assert c.results.get('matches') > 0



from owslib.wps import WebProcessingService
import owslib.wps
from owslib.etree import etree


def test_wps_getcapabilities_language(monkeypatch):

    def mock_open_url(*args, **kwargs):
        assert 'language=fr-CA' in args[1]

        class FakeResponse:
            def read(self):
                return b'<xml></xml>'

        return FakeResponse()

    monkeypatch.setattr(owslib.wps, "openURL", mock_open_url)
    WebProcessingService('http://www.example.com', language='fr-CA')


def test_wps_getcapabilities_parse_languages():
    xml = """
    <wps:Languages xmlns:wps="http://www.opengis.net/wps/1.0.0" xmlns:ows="http://www.opengis.net/ows">
        <wps:Default>
            <ows:Language>en-US</ows:Language>
        </wps:Default>
        <wps:Supported>
            <ows:Language>en-US</ows:Language>
            <ows:Language>fr-CA</ows:Language>
        </wps:Supported>
    </wps:Languages>
    """
    element = etree.fromstring(xml)
    languages = owslib.wps.Languages(element)
    assert languages.default == 'en-US'
    assert languages.supported == ['en-US', 'fr-CA']



# -*- coding: utf-8 -*-

import io

from owslib import util
from owslib.etree import etree
from owslib.iso import (
    MD_Metadata, Keyword
)
from owslib.namespaces import Namespaces


def get_md_resource(file_path):
    """Read the file and parse into an XML tree.

    Parameters
    ----------
    file_path : str
        Path of the file to read.

    Returns
    -------
    etree.ElementTree
        XML tree of the resource on disk.

    """
    namespaces = Namespaces().get_namespaces(keys=('gmd', 'gmi'))

    with io.open(file_path, mode='r', encoding='utf-8') as f:
        data = f.read().encode('utf-8')
        data = etree.fromstring(data)
        mdelem = data.find('.//' + util.nspath_eval(
            'gmd:MD_Metadata', namespaces))

        if mdelem is None:
            mdelem = data.find(
                './/' + util.nspath_eval('gmi:MI_Metadata', namespaces))

        if mdelem is None and data.tag in ['{http://www.isotc211.org/2005/gmd}MD_Metadata',
                                           '{http://www.isotc211.org/2005/gmi}MI_Metadata']:
            mdelem = data

    return mdelem


def assert_list(var, length):
    """Assert a given variable is a list with given size.

    Parameters
    ----------
    var : variable
        Variable to test (i.e. should be a list).
    length : int
        The length/size of the list.

    """
    assert type(var) is list
    assert len(var) == length


def test_md_parsing_dov():
    """Test the parsing of a metadatarecord from DOV

    GetRecordById response available in
    tests/resources/csw_dov_getrecordbyid.xml

    """
    md_resource = get_md_resource('tests/resources/csw_dov_getrecordbyid.xml')
    md = MD_Metadata(md_resource)

    assert type(md) is MD_Metadata

    assert md.identifier == '6c39d716-aecc-4fbc-bac8-4f05a49a78d5'
    assert md.dataseturi is None
    assert md.parentidentifier is None

    assert md.language is None
    assert md.languagecode == 'dut'

    assert md.charset == 'utf8'
    assert md.datestamp == '2018-02-21T16:14:24'

    assert md.hierarchy == 'dataset'

    assert_list(md.contact, 1)

    contact = md.contact[0]
    assert contact.organization == 'Vlaamse overheid - Vlaamse ' \
                                   'MilieuMaatschappij - Afdeling ' \
                                   'Operationeel Waterbeheer'
    assert contact.address == 'Koning Albert II-laan 20 bus 16'
    assert contact.city == 'Brussel'
    assert contact.postcode == '1000'
    assert contact.country == u'België'
    assert contact.email == 'info@vmm.be'
    assert contact.onlineresource.url == 'https://www.vmm.be'
    assert contact.role == 'pointOfContact'

    assert md.stdname == 'ISO 19115/2003/Cor.1:2006'
    assert md.stdver == 'GDI-Vlaanderen Best Practices - versie 1.0'

    assert md.referencesystem.code == '31370'
    assert md.referencesystem.codeSpace == 'EPSG'

    assert_list(md.identificationinfo, 1)

    iden = md.identificationinfo[0]
    assert iden.title == 'Grondwatermeetnetten'
    assert iden.alternatetitle == 'Grondwatermeetnetten beschikbaar op DOV'

    assert_list(iden.date, 2)
    assert iden.date[0].date == '2002-05-22'
    assert iden.date[0].type == 'creation'
    assert iden.date[1].date == '2002-05-22'
    assert iden.date[1].type == 'publication'

    assert_list(iden.uricode, 1)
    assert iden.uricode[0] == 'A64F073B-9FBE-91DD-36FDE7462BBAFA61'

    assert_list(iden.uricodespace, 1)
    assert iden.uricodespace[0] == 'DOV-be'

    assert_list(iden.uselimitation, 3)
    assert "Zie 'Overige beperkingen'" in iden.uselimitation
    assert "Bij het gebruik van de informatie die DOV aanbiedt, dient steeds " \
           "volgende standaardreferentie gebruikt te worden: Databank " \
           "Ondergrond Vlaanderen - (vermelding van de beheerder en de " \
           "specifieke geraadpleegde gegevens) - Geraadpleegd op dd/mm/jjjj, " \
           "op https://www.dov.vlaanderen.be" in iden.uselimitation
    assert "Volgende aansprakelijkheidsbepalingen gelden: " \
           "https://www.dov.vlaanderen.be/page/disclaimer" in iden.uselimitation

    assert_list(iden.uselimitation_url, 0)

    assert_list(iden.accessconstraints, 1)
    assert iden.accessconstraints[0] == 'otherRestrictions'

    assert_list(iden.classification, 0)

    assert_list(iden.otherconstraints, 1)
    assert iden.otherconstraints[
               0] == "Data beschikbaar voor hergebruik volgens de " \
                     "Modellicentie Gratis Hergebruik. Toelichting " \
                     "beschikbaar op " \
                     "https://www.dov.vlaanderen.be/page/gebruiksvoorwaarden-dov-services"

    assert_list(iden.securityconstraints, 1)
    assert iden.securityconstraints[0] == 'unclassified'

    assert_list(iden.useconstraints, 0)

    assert_list(iden.denominators, 1)
    assert iden.denominators[0] == '10000'

    assert_list(iden.distance, 0)
    assert_list(iden.uom, 0)

    assert_list(iden.resourcelanguage, 0)
    assert_list(iden.resourcelanguagecode, 1)
    assert iden.resourcelanguagecode[0] == 'dut'

    assert_list(iden.creator, 0)
    assert_list(iden.publisher, 0)
    assert_list(iden.contributor, 0)

    assert iden.edition is None

    assert iden.abstract.startswith("In de Databank Ondergrond Vlaanderen "
                                    "zijn verschillende grondwatermeetnetten "
                                    "opgenomen.")

    assert iden.purpose.startswith(
        "Het doel van de meetnetten is inzicht krijgen in de kwaliteit en "
        "kwantiteit van de watervoerende lagen in de ondergrond van "
        "Vlaanderen. Algemeen kan gesteld worden dat de grondwatermeetnetten "
        "een belangrijk beleidsinstrument vormen")

    assert iden.status == 'onGoing'

    assert_list(iden.contact, 2)

    assert iden.contact[0].organization == 'Vlaamse overheid - Vlaamse MilieuMaatschappij - Afdeling Operationeel Waterbeheer'
    assert iden.contact[0].address == 'Koning Albert II-laan 20 bus 16'
    assert iden.contact[0].city == 'Brussel'
    assert iden.contact[0].postcode == '1000'
    assert iden.contact[0].country == u'België'
    assert iden.contact[0].email == 'info@vmm.be'
    assert iden.contact[0].onlineresource.url == 'https://www.vmm.be'
    assert iden.contact[0].role == 'pointOfContact'

    assert iden.contact[1].organization == 'Databank Ondergrond Vlaanderen (' \
                                           'DOV)'
    assert iden.contact[1].address == 'Technologiepark Gebouw 905'
    assert iden.contact[1].city == 'Zwijnaarde'
    assert iden.contact[1].postcode == '9052'
    assert iden.contact[1].country == u'België'
    assert iden.contact[1].email == 'dov@vlaanderen.be'
    assert iden.contact[1].onlineresource.url == \
           'https://www.dov.vlaanderen.be'
    assert iden.contact[1].role == 'distributor'

    assert_list(iden.spatialrepresentationtype, 1)
    assert iden.spatialrepresentationtype[0] == 'vector'

    assert_list(iden.keywords, 5)

    assert type(iden.keywords[0]) is dict
    assert iden.keywords[0]['type'] == ''
    assert iden.keywords[0]['thesaurus']['title'] == "GEMET - INSPIRE thema's, versie 1.0"
    assert iden.keywords[0]['thesaurus']['date'] == '2008-06-01'
    assert iden.keywords[0]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[0]['keywords'], 1)
    assert iden.keywords[0]['keywords'] == ['Geologie']

    assert type(iden.keywords[1]) is dict
    assert iden.keywords[1]['type'] == ''
    assert iden.keywords[1]['thesaurus'][
               'title'] == "GEMET - Concepten, versie 2.4"
    assert iden.keywords[1]['thesaurus']['date'] == '2010-01-13'
    assert iden.keywords[1]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[1]['keywords'], 2)
    assert iden.keywords[1]['keywords'] == ['grondwater', 'meetnet(werk)']

    assert type(iden.keywords[2]) is dict
    assert iden.keywords[2]['type'] == ''
    assert iden.keywords[2]['thesaurus'][
               'title'] == "Vlaamse regio's"
    assert iden.keywords[2]['thesaurus']['date'] == '2013-09-25'
    assert iden.keywords[2]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[2]['keywords'], 1)
    assert iden.keywords[2]['keywords'] == ['Vlaams Gewest']

    assert type(iden.keywords[3]) is dict
    assert iden.keywords[3]['type'] is None
    assert iden.keywords[3]['thesaurus'][
               'title'] == "GDI-Vlaanderen Trefwoorden"
    assert iden.keywords[3]['thesaurus']['date'] == '2014-02-26'
    assert iden.keywords[3]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[3]['keywords'], 7)
    assert iden.keywords[3]['keywords'] == [
        'Toegevoegd GDI-Vl', 'Herbruikbaar', 'Vlaamse Open data',
        'Kosteloos', 'Lijst M&R INSPIRE', 'Metadata INSPIRE-conform',
        'Metadata GDI-Vl-conform']

    assert type(iden.keywords[4]) is dict
    assert iden.keywords[4]['type'] is None
    assert iden.keywords[4]['thesaurus']['title'] == "DOV"
    assert iden.keywords[4]['thesaurus']['date'] == '2010-12-01'
    assert iden.keywords[4]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[4]['keywords'], 7)
    assert iden.keywords[4]['keywords'] == [
        'Ondergrond', 'DOV', 'Vlaanderen', 'monitoring', 'meetnetten',
        'Kaderrichtlijn Water', 'Decreet Integraal waterbeleid']

    assert_list(iden.keywords2, 5)
    assert iden.keywords2[0].type == ''
    assert iden.keywords2[0].thesaurus[
               'title'] == "GEMET - INSPIRE thema's, versie 1.0"
    assert iden.keywords2[0].thesaurus['date'] == '2008-06-01'
    assert iden.keywords2[0].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[0].keywords, 1)
    assert iden.keywords2[0].keywords == ['Geologie']

    assert iden.keywords2[1].type == ''
    assert iden.keywords2[1].thesaurus[
               'title'] == "GEMET - Concepten, versie 2.4"
    assert iden.keywords2[1].thesaurus['date'] == '2010-01-13'
    assert iden.keywords2[1].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[1].keywords, 2)
    assert iden.keywords2[1].keywords == ['grondwater', 'meetnet(werk)']

    assert iden.keywords2[2].type == ''
    assert iden.keywords2[2].thesaurus[
               'title'] == "Vlaamse regio's"
    assert iden.keywords2[2].thesaurus['date'] == '2013-09-25'
    assert iden.keywords2[2].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[2].keywords, 1)
    assert iden.keywords2[2].keywords == ['Vlaams Gewest']

    assert iden.keywords2[3].type is None
    assert iden.keywords2[3].thesaurus[
               'title'] == "GDI-Vlaanderen Trefwoorden"
    assert iden.keywords2[3].thesaurus['date'] == '2014-02-26'
    assert iden.keywords2[3].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[3].keywords, 7)
    assert iden.keywords2[3].keywords == [
        'Toegevoegd GDI-Vl', 'Herbruikbaar', 'Vlaamse Open data',
        'Kosteloos', 'Lijst M&R INSPIRE', 'Metadata INSPIRE-conform',
        'Metadata GDI-Vl-conform']

    assert iden.keywords2[4].type is None
    assert iden.keywords2[4].thesaurus['title'] == "DOV"
    assert iden.keywords2[4].thesaurus['date'] == '2010-12-01'
    assert iden.keywords2[4].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[4].keywords, 7)
    assert iden.keywords2[4].keywords == [
        'Ondergrond', 'DOV', 'Vlaanderen', 'monitoring', 'meetnetten',
        'Kaderrichtlijn Water', 'Decreet Integraal waterbeleid']

    assert_list(iden.topiccategory, 1)
    assert iden.topiccategory[0] == 'geoscientificInformation'

    assert iden.supplementalinformation == \
           "https://www.dov.vlaanderen.be/page/grondwatermeetnet"

    assert_list(md.contentinfo, 1)
    ci = md.contentinfo[0]

    assert ci.compliancecode is None
    assert_list(ci.language, 0)
    assert ci.includedwithdataset == True
    assert_list(ci.featuretypenames, 0)

    assert_list(ci.featurecatalogues, 1)
    assert ci.featurecatalogues[0] == 'b142965f-b2aa-429e-86ff-a7cb0e065d48'


def test_md_parsing_geobretagne():
    """Test the parsing of a metadatarecord from GéoBretagne

    MD_Metadata record available in
    tests/resources/csw_geobretagne_mdmetadata.xml

    """
    md_resource = get_md_resource(
        'tests/resources/csw_geobretagne_mdmetadata.xml')
    md = MD_Metadata(md_resource)

    assert type(md) is MD_Metadata

    assert md.identifier == '955c3e47-411e-4969-b61b-3556d1b9f879'
    assert md.dataseturi is None
    assert md.parentidentifier is None

    assert md.language == 'fre'
    assert md.languagecode is None

    assert md.charset == 'utf8'
    assert md.datestamp == '2018-07-30T14:19:40'

    assert md.hierarchy == 'dataset'

    assert_list(md.contact, 1)

    contact = md.contact[0]
    assert contact.organization == 'DIRECTION GENERALE DES FINANCES ' \
                                   'PUBLIQUES BUREAU GF-3A'
    assert contact.address is None
    assert contact.city is None
    assert contact.postcode is None
    assert contact.country is None
    assert contact.email == 'bureau.gf3a@dgfip.finances.gouv.fr'
    assert contact.onlineresource is None
    assert contact.role == 'pointOfContact'

    assert md.stdname == 'ISO 19115'
    assert md.stdver == '1.0'

    assert md.referencesystem.code == 'RGF93 / CC48 (EPSG:3948)'
    assert md.referencesystem.codeSpace == 'EPSG'

    assert_list(md.identificationinfo, 1)

    iden = md.identificationinfo[0]
    assert iden.title == 'Cadastre 2018 en Bretagne'
    assert iden.alternatetitle is None

    assert_list(iden.date, 1)
    assert iden.date[0].date == '2018-09-01'
    assert iden.date[0].type == 'revision'

    assert_list(iden.uricode, 1)
    assert iden.uricode[0] == 'https://geobretagne.fr/geonetwork/apps/georchestra/?uuid=363e3a8e-d0ce-497d-87a9-2a2d58d82772'
    assert_list(iden.uricodespace, 0)

    assert_list(iden.uselimitation, 2)
    assert u"le plan cadastral décrit les limites apparentes de la " \
           u"propriété." in iden.uselimitation

    assert_list(iden.uselimitation_url, 0)

    assert_list(iden.accessconstraints, 1)
    assert iden.accessconstraints[0] == 'otherRestrictions'

    assert_list(iden.classification, 0)

    assert_list(iden.otherconstraints, 1)
    assert iden.otherconstraints[
               0] == u'Usage libre sous réserve des mentions obligatoires ' \
                     u'sur tout document de diffusion : "Source : DGFIP"'

    assert_list(iden.securityconstraints, 0)

    assert_list(iden.useconstraints, 1)
    assert iden.useconstraints[0] == 'copyright'

    assert_list(iden.denominators, 1)
    assert iden.denominators[0] == '500'

    assert_list(iden.distance, 0)
    assert_list(iden.uom, 0)

    assert_list(iden.resourcelanguage, 1)
    assert iden.resourcelanguage[0] == 'fre'
    assert_list(iden.resourcelanguagecode, 0)

    assert_list(iden.creator, 0)
    assert_list(iden.publisher, 0)
    assert_list(iden.contributor, 0)

    assert iden.edition is None

    assert iden.abstract.startswith(
        u"Le plan du cadastre est un document administratif qui propose "
        u"l’unique plan parcellaire à grande échelle couvrant le territoire "
        u"national.")

    assert iden.purpose.startswith(
        u"Le but premier du plan cadastral est d'identifier, de localiser et "
        u"représenter la propriété foncière, ainsi que de servir à l'assise "
        u"de la fiscalité locale des propriétés non bâties.")

    assert iden.status == 'completed'

    assert_list(iden.contact, 1)

    assert iden.contact[0].organization == 'DGFIP Bretagne'
    assert iden.contact[0].name == 'DIRECTION GENERALE DES FINANCES PUBLIQUES'
    assert iden.contact[0].address is None
    assert iden.contact[0].city is None
    assert iden.contact[0].postcode is None
    assert iden.contact[0].country is None
    assert iden.contact[0].email == 'bureau.gf3a@dgfip.finances.gouv.fr'
    assert iden.contact[0].onlineresource is None
    assert iden.contact[0].role == 'pointOfContact'

    assert_list(iden.spatialrepresentationtype, 1)
    assert iden.spatialrepresentationtype[0] == 'vector'

    assert_list(iden.keywords, 7)

    assert type(iden.keywords[0]) is dict
    assert iden.keywords[0]['type'] == 'place'
    assert iden.keywords[0]['thesaurus']['title'] is None
    assert iden.keywords[0]['thesaurus']['date'] is None
    assert iden.keywords[0]['thesaurus']['datetype'] is None
    assert_list(iden.keywords[0]['keywords'], 1)
    assert iden.keywords[0]['keywords'] == ['France']

    assert type(iden.keywords[1]) is dict
    assert iden.keywords[1]['type'] is None
    assert iden.keywords[1]['thesaurus']['title'] is None
    assert iden.keywords[1]['thesaurus']['date'] is None
    assert iden.keywords[1]['thesaurus']['datetype'] is None
    assert_list(iden.keywords[1]['keywords'], 0)

    assert type(iden.keywords[2]) is dict
    assert iden.keywords[2]['type'] == 'theme'
    assert iden.keywords[2]['thesaurus']['title'] is None
    assert iden.keywords[2]['thesaurus']['date'] is None
    assert iden.keywords[2]['thesaurus']['datetype'] is None
    assert_list(iden.keywords[2]['keywords'], 7)
    assert iden.keywords[2]['keywords'] == [
        u'bâtiments', 'adresses', 'parcelles cadastrales', 'hydrographie',
        u'réseaux de transport', u'unités administratives',
        u'référentiels de coordonnées']

    assert type(iden.keywords[3]) is dict
    assert iden.keywords[3]['type'] == 'theme'
    assert iden.keywords[3]['thesaurus']['title'] is None
    assert iden.keywords[3]['thesaurus']['date'] is None
    assert iden.keywords[3]['thesaurus']['datetype'] is None
    assert_list(iden.keywords[3]['keywords'], 5)
    assert iden.keywords[3]['keywords'] == [
        u'bâtis', 'sections', 'parcelles', 'cadastre', 'cadastrale']

    assert type(iden.keywords[4]) is dict
    assert iden.keywords[4]['type'] == 'theme'
    assert iden.keywords[4]['thesaurus']['title'] == u"GéoBretagne v 2.0"
    assert iden.keywords[4]['thesaurus']['date'] == '2014-01-13'
    assert iden.keywords[4]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[4]['keywords'], 1)
    assert iden.keywords[4]['keywords'] == [u'référentiels : cadastre']

    assert type(iden.keywords[5]) is dict
    assert iden.keywords[5]['type'] == 'theme'
    assert iden.keywords[5]['thesaurus']['title'] == "INSPIRE themes"
    assert iden.keywords[5]['thesaurus']['date'] == '2008-06-01'
    assert iden.keywords[5]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[5]['keywords'], 1)
    assert iden.keywords[5]['keywords'] == ['Parcelles cadastrales']

    assert type(iden.keywords[6]) is dict
    assert iden.keywords[6]['type'] == 'theme'
    assert iden.keywords[6]['thesaurus']['title'] == "GEMET"
    assert iden.keywords[6]['thesaurus']['date'] == '2012-07-20'
    assert iden.keywords[6]['thesaurus']['datetype'] == 'publication'
    assert_list(iden.keywords[6]['keywords'], 2)
    assert iden.keywords[6]['keywords'] == ['cadastre', u'bâtiment']

    assert_list(iden.keywords2, 6)

    assert iden.keywords2[0].type == 'place'
    assert iden.keywords2[0].thesaurus is None
    assert_list(iden.keywords2[0].keywords, 1)
    assert iden.keywords2[0].keywords == ['France']

    assert iden.keywords2[1].type == 'theme'
    assert iden.keywords2[1].thesaurus is None
    assert_list(iden.keywords2[1].keywords, 7)
    assert iden.keywords2[1].keywords == [
        u'bâtiments', 'adresses', 'parcelles cadastrales', 'hydrographie',
        u'réseaux de transport', u'unités administratives',
        u'référentiels de coordonnées']

    assert iden.keywords2[2].type == 'theme'
    assert iden.keywords2[2].thesaurus is None
    assert_list(iden.keywords2[2].keywords, 5)
    assert iden.keywords2[2].keywords == [
        u'bâtis', 'sections', 'parcelles', 'cadastre', 'cadastrale']

    assert iden.keywords2[3].type == 'theme'
    assert iden.keywords2[3].thesaurus['title'] == u"GéoBretagne v 2.0"
    assert iden.keywords2[3].thesaurus['date'] == '2014-01-13'
    assert iden.keywords2[3].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[3].keywords, 1)
    assert iden.keywords2[3].keywords == [u'référentiels : cadastre']

    assert iden.keywords2[4].type == 'theme'
    assert iden.keywords2[4].thesaurus['title'] == "INSPIRE themes"
    assert iden.keywords2[4].thesaurus['date'] == '2008-06-01'
    assert iden.keywords2[4].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[4].keywords, 1)
    assert iden.keywords2[4].keywords == ['Parcelles cadastrales']

    assert iden.keywords2[5].type == 'theme'
    assert iden.keywords2[5].thesaurus['title'] == "GEMET"
    assert iden.keywords2[5].thesaurus['date'] == '2012-07-20'
    assert iden.keywords2[5].thesaurus['datetype'] == 'publication'
    assert_list(iden.keywords2[5].keywords, 2)
    assert iden.keywords2[5].keywords == ['cadastre', u'bâtiment']

    assert_list(iden.topiccategory, 1)
    assert iden.topiccategory[0] == 'planningCadastre'

    assert iden.supplementalinformation == \
           u"La légende du plan cadastral est consultable sur: " \
           "http://www.cadastre.gouv.fr/scpc/pdf/legendes/FR_fr/Legende%20du" \
           "%20plan%20sur%20internet.pdf"

    assert_list(md.contentinfo, 1)
    ci = md.contentinfo[0]

    assert ci.compliancecode is None
    assert_list(ci.language, 0)
    assert ci.includedwithdataset == False
    assert_list(ci.featuretypenames, 0)
    assert_list(ci.featurecatalogues, 0)


def test_md_parsing_19115_2():
    """Test the parsing of a 19115-2 document

    MD_Metadata record available in
    tests/resources/iso_mi.xml

    """
    md_resource = get_md_resource(
        'tests/resources/iso_mi.xml')
    md = MD_Metadata(md_resource)

    assert type(md) is MD_Metadata

    assert md.identifier == '3f342f64-9348-11df-ba6a-0014c2c00eab'

    iden = md.identificationinfo[0]

    assert len(iden.keywords2) == 3
    assert iden.keywords2[1].thesaurus['title'] == 'My Vocabulary'
    assert iden.keywords2[1].thesaurus['url'] == 'https://example.org/my-vocab'

    ci = md.contentinfo[0]
    assert ci.type == 'image'
    assert ci.cloud_cover == '72'
    assert ci.processing_level == '1.0'

    band = ci.bands[0]
    assert band.id == 'B1'
    assert band.units == 'nm'
    assert band.min == '932'
    assert band.max == '958'

    plt = md.acquisition.platforms[0]
    assert plt.identifier == 'LANDSAT_8'
    assert plt.description == 'Landsat 8'

    inst = plt.instruments[0]
    assert inst.identifier == 'OLI_TIRS'
    assert inst.type == 'INS-NOBS'

def test_md_parsing_keywords_anchor():
    """Test the parsing of MD_Keywords where the keyword is defined by a 
    gmx:Anchor
    
    MD_Metadata record available in
    tests/resources/iso_keywords_anchor.xml
    
    """
    md_resource = get_md_resource(
        'tests/resources/iso_keywords_anchor.xml')
    md = MD_Metadata(md_resource)
    
    assert type(md) is MD_Metadata
    assert md.identifier == 'ie.marine.data:dataset.1135'
    
    iden = md.identificationinfo[0]
    assert len(iden.keywords2) == 1
    assert len(iden.keywords2[0].keywords_object) == 6
    assert type(iden.keywords2[0].keywords_object[0]) is Keyword
    assert iden.keywords2[0].keywords_object[0].name == 'Atmospheric pressure'
    assert iden.keywords2[0].keywords_object[0].url == 'http://vocab.nerc.ac.uk/collection/A05/current/EV_AIRPRESS/'
    assert iden.keywords2[0].thesaurus['title'] == 'AtlantOS Essential Variables'
    assert iden.keywords2[0].thesaurus['url'] == 'http://vocab.nerc.ac.uk/collection/A05/current/'
    
def test_md_parsing_keywords_no_anchor():
    """Test the parsing of MD_Keywords where the keyword is not defined by a 
    gmx:Anchor
    
    MD_Metadata record available in
    tests/resources/csw_geobretagne_mdmetadata.xml

    """
    md_resource = get_md_resource(
        'tests/resources/csw_geobretagne_mdmetadata.xml')
    md = MD_Metadata(md_resource)
    assert type(md) is MD_Metadata
    
    iden = md.identificationinfo[0]
    assert len(iden.keywords2) == 6
    assert len(iden.keywords2[0].keywords_object) == 1
    assert type(iden.keywords2[0].keywords_object[0]) is Keyword
    assert iden.keywords2[0].keywords_object[0].name == 'France'
    assert iden.keywords2[0].keywords_object[0].url is None
    
    assert len(iden.keywords2[1].keywords_object) == 7
    assert iden.keywords2[1].keywords_object[0].name == 'bâtiments'
    assert iden.keywords2[1].keywords_object[0].url is None
    assert iden.keywords2[1].keywords_object[1].name == 'adresses'
    assert iden.keywords2[1].keywords_object[1].url is None
    assert iden.keywords2[1].keywords_object[2].name == 'parcelles cadastrales'
    assert iden.keywords2[1].keywords_object[2].url is None



# Test generation of a WPS request from input arguments.
# The specific request involves a "bbox" process submitted to the Emu WPS service
# (https://github.com/bird-house/emu).

from tests.utils import resource_file, compare_xml
from owslib.wps import WebProcessingService, WPSExecution, BoundingBoxDataInput
from owslib.etree import etree


def test_wps_request11_bbox():
    processid = "bbox"
    bbox = BoundingBoxDataInput([51.9, 7.0, 53.0, 8.0])
    inputs = [("bbox", bbox)]

    # Build XML request for WPS process execution
    execution = WPSExecution()
    requestElement = execution.buildRequest(processid, inputs)
    request = etree.tostring(requestElement)

    # Compare to cached XML request
    _request = open(resource_file('wps_EmuExecuteRequest11.xml'), 'rb').read()
    assert compare_xml(request, _request) is True



# Test generation of a WPS request from input arguments.
# The specific request involves a "reprojectImage" process submitted to the PML WPS service.

from tests.utils import resource_file, compare_xml
from owslib.wps import WebProcessingService, WPSExecution
from owslib.etree import etree


def test_wps_request4():
    # Process input/ouutput arguments
    processid = "reprojectImage"
    inputs = [("inputImage", "http://rsg.pml.ac.uk/wps/testdata/elev_srtm_30m.img"),
              ("outputSRS", "EPSG:4326")]
    output = "outputImage"

    # build XML request for WPS process execution
    execution = WPSExecution()
    requestElement = execution.buildRequest(processid, inputs, output=[(output, True)])
    request = etree.tostring(requestElement)

    # Compare to cached XML request
    _request = open(resource_file('wps_PMLExecuteRequest4.xml'), 'rb').read()
    assert compare_xml(request, _request) is True



#!/usr/bin/python
# -*- coding: ISO-8859-15 -*-
# =============================================================================
# Copyright (c) 2010 Tom Kralidis
#
# Authors : Tom Kralidis <tomkralidis@gmail.com>
#
# Contact email: tomkralidis@gmail.com
# =============================================================================

# simple process to harvest CSW catalogues via Harvest operations

import sys
from owslib.csw import CatalogueServiceWeb

stop = 0
flag = 0
maxrecords = 10

if len(sys.argv) < 3:
    print('Usage: %s <source_catalogue_url> <destination_catalogue_url> [maxrecords]' \
        % sys.argv[0])
    sys.exit(1)

src = CatalogueServiceWeb(sys.argv[1])
dest = CatalogueServiceWeb(sys.argv[2])

if len(sys.argv) == 4:
    maxrecords = sys.argv[3]

while stop == 0:
    if flag == 0:  # first run, start from 0
        startposition = 0
    else:  # subsequent run, startposition is now paged
        startposition = src.results['nextrecord']

    src.getrecords(esn='brief', startposition=startposition, maxrecords=maxrecords)

    print(src.results)

    if src.results['nextrecord'] == 0 \
        or src.results['returned'] == 0 \
        or src.results['nextrecord'] > src.results['matches']:  # end the loop, exhausted all records
        stop = 1
        break

    # harvest each record to destination CSW
    for i in list(src.records):
        source = '%s?service=CSW&version=2.0.2&request=GetRecordById&id=%s' % \
            (sys.argv[1], i)
        dest.harvest(source=source, \
            resourcetype='http://www.isotc211.org/2005/gmd')
        #print dest.request
        #print dest.response

    flag = 1



# Simulate a WPS Execute invocation.
# This test does not execute any live HTTP request, rather it parses XML files containing pre-made HTTP responses.

from tests.utils import resource_file
from owslib.wps import WebProcessingService


def test_wps_execute():
    wps = WebProcessingService(
        'http://cida.usgs.gov/gdp/process/WebProcessingService',
        skip_caps=True)

    # Execute fake invocation of Execute operation using cached HTTP request and response
    request = open(resource_file('wps_USGSExecuteRequest1.xml'), 'rb').read()
    response = open(resource_file('wps_USGSExecuteResponse1a.xml'), 'rb').read()
    execution = wps.execute(None, [], request=request, response=response)
    assert execution.status == 'ProcessStarted'
    assert execution.isComplete() is False

    # Simulate end of process
    response = open(resource_file('wps_USGSExecuteResponse1b.xml'), 'rb').read()
    execution.checkStatus(sleepSecs=0, response=response)
    assert execution.status == 'ProcessSucceeded'
    assert execution.isComplete() is True

    # Display location of process output
    output = execution.processOutputs[0]
    assert output.reference == \
        'http://cida.usgs.gov/climate/gdp/process/RetrieveResultServlet?id=1318528582026OUTPUT.601bb3d0-547f-4eab-8642-7c7d2834459e'  # noqa



from tests.utils import service_ok

import pytest

from owslib.ogcapi.features import Features

SERVICE_URL = 'https://demo.pygeoapi.io/master'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason='service is unreachable')
def test_ogcapi_features_pygeoapi():
    w = Features(SERVICE_URL)

    assert w.url == 'https://demo.pygeoapi.io/master/'
    assert w.url_query_string is None

    api = w.api()
    assert api['components']['parameters'] is not None
    paths = api['paths']
    assert paths is not None
    assert paths['/collections/lakes'] is not None

    conformance = w.conformance()
    assert len(conformance['conformsTo']) > 0

    collections = w.collections()
    assert len(collections) > 0

    feature_collections = w.feature_collections()
    assert len(feature_collections) > 0

    lakes = w.collection('lakes')
    assert lakes['id'] == 'lakes'
    assert lakes['title'] == 'Large Lakes'
    assert lakes['description'] == 'lakes of the world, public domain'

    #lakes_queryables = w.collection_queryables('lakes')
    #assert len(lakes_queryables['queryables']) == 6

    # Minimum of limit param is 1
    with pytest.raises(RuntimeError):
        lakes_query = w.collection_items('lakes', limit=0)

    lakes_query = w.collection_items('lakes', limit=1, admin='admin-0')
    assert lakes_query['numberMatched'] > 0
    assert lakes_query['numberReturned'] == 1
    assert len(lakes_query['features']) == 1



# This test checks that the OWSLib service interfaces are the same across all service type:
# Author: Dominic Lowe, 17th September 2009
# Part of OWSLib package.
from tests.utils import service_ok

import pytest

from tests.utils import resource_file
import owslib
from owslib.csw import CatalogueServiceWeb
from owslib.wms import WebMapService
from owslib.wcs import WebCoverageService
from owslib.wfs import WebFeatureService
from owslib.util import OrderedDict


# TODO, we should run all these from local XML documents (as per the WMS and WFS services)
# CSW_SERVICE_URL = 'http://data.nodc.noaa.gov/geoportal/csw'
CSW_SERVICE_URL = 'https://demo.pycsw.org/cite/csw'
WCS_SERVICE_URL = 'http://thredds.ucar.edu/thredds/wcs/grib/NCEP/NAM/CONUS_80km/best'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(CSW_SERVICE_URL),
                    reason='service is unreachable')
def test_ows_interfaces_csw():
    service = CatalogueServiceWeb(CSW_SERVICE_URL)
    # Check each service instance conforms to OWSLib interface
    service.alias = 'CSW'
    isinstance(service, owslib.catalogue.csw2.CatalogueServiceWeb)
    # URL attribute
    assert service.url == CSW_SERVICE_URL
    # version attribute
    assert service.version == '2.0.2'
    # Identification object
    assert hasattr(service, 'identification')
    # Check all ServiceIdentification attributes
    assert service.identification.type == 'CSW'
    for attribute in ['type', 'version', 'title', 'abstract', 'keywords', 'accessconstraints', 'fees']:
        assert hasattr(service.identification, attribute)
    # Check all ServiceProvider attributes
    for attribute in ['name', 'url', 'contact']:
        assert hasattr(service.provider, attribute)
    # Check all operations implement IOperationMetadata
    for op in service.operations:
        for attribute in ['name', 'formatOptions', 'methods']:
            assert hasattr(op, attribute)
    # Check all contents implement IContentMetadata as a dictionary
    # CSW does not work in this way so use dummy
    service.contents = {'dummy': '1'}
    isinstance(service.contents, dict)
    # Check any item (WCS coverage, WMS layer etc) from the contents of each service
    # Check it conforms to IContentMetadata interface
    # CSW does not conform to this


def test_ows_interfaces_wms():
    wmsxml = open(resource_file('wms_JPLCapabilities.xml'), 'rb').read()
    service = WebMapService('url', version='1.1.1', xml=wmsxml)
    # Check each service instance conforms to OWSLib interface
    service.alias = 'WMS'
    isinstance(service, owslib.map.wms111.WebMapService_1_1_1)
    # URL attribute
    assert service.url == 'url'
    # version attribute
    assert service.version == '1.1.1'
    # Identification object
    assert hasattr(service, 'identification')
    # Check all ServiceIdentification attributes
    assert service.identification.type == 'OGC:WMS'
    for attribute in ['type', 'version', 'title', 'abstract', 'keywords', 'accessconstraints', 'fees']:
        assert hasattr(service.identification, attribute)
    # Check all ServiceProvider attributes
    for attribute in ['name', 'url', 'contact']:
        assert hasattr(service.provider, attribute)
    # Check all operations implement IOperationMetadata
    for op in service.operations:
        for attribute in ['name', 'formatOptions', 'methods']:
            assert hasattr(op, attribute)
    # Check all contents implement IContentMetadata as a dictionary
    isinstance(service.contents, OrderedDict)
    # Check any item (WCS coverage, WMS layer etc) from the contents of each service
    # Check it conforms to IContentMetadata interface
    # get random item from contents dictionary -has to be a nicer way to do this!
    content = service.contents[list(service.contents.keys())[0]]
    for attribute in ['id', 'title', 'boundingBox', 'boundingBoxWGS84', 'crsOptions', 'styles', 'timepositions']:
        assert hasattr(content, attribute)


@pytest.mark.online
def test_ows_interfaces_wcs():
    service = WebCoverageService(WCS_SERVICE_URL, version='1.0.0')
    # Check each service instance conforms to OWSLib interface
    service.alias = 'WCS'
    isinstance(service, owslib.coverage.wcs100.WebCoverageService_1_0_0)
    # URL attribute
    assert service.url == WCS_SERVICE_URL
    # version attribute
    assert service.version == '1.0.0'
    # Identification object
    assert hasattr(service, 'identification')
    # Check all ServiceIdentification attributes
    assert service.identification.type == 'OGC:WCS'
    for attribute in ['type', 'version', 'title', 'abstract', 'keywords', 'fees']:
        assert hasattr(service.identification, attribute)
    # Check all ServiceProvider attributes
    for attribute in ['name', 'url', 'contact']:
        assert hasattr(service.provider, attribute)
    # Check all operations implement IOperationMetadata
    for op in service.operations:
        for attribute in ['name', 'methods']:
            assert hasattr(op, attribute)
    # Check all contents implement IContentMetadata as a dictionary
    isinstance(service.contents, dict)
    # Check any item (WCS coverage, WMS layer etc) from the contents of each service
    # Check it conforms to IContentMetadata interface
    # get random item from contents dictionary -has to be a nicer way to do this!
    content = service.contents[list(service.contents.keys())[0]]
    for attribute in ['id', 'title', 'boundingBox', 'boundingBoxWGS84', 'crsOptions', 'styles', 'timepositions']:
        assert hasattr(content, attribute)


def test_ows_interfaces_wfs():
    wfsxml = open(resource_file('mapserver-wfs-cap.xml'), 'rb').read()
    service = WebFeatureService('url', version='1.0', xml=wfsxml)
    # Check each service instance conforms to OWSLib interface
    service.alias = 'CSW'
    isinstance(service, owslib.feature.wfs100.WebFeatureService_1_0_0)
    # URL attribute
    assert service.url == 'url'
    # version attribute
    assert service.version == '1.0'
    # Identification object
    assert hasattr(service, 'identification')
    # Check all ServiceIdentification attributes
    assert service.identification.type == 'MapServer WFS'
    for attribute in ['type', 'version', 'title', 'abstract', 'keywords', 'accessconstraints', 'fees']:
        assert hasattr(service.identification, attribute)
    # Check all ServiceProvider attributes
    for attribute in ['name', 'url']:
        assert hasattr(service.provider, attribute)
    # Check all operations implement IOperationMetadata
    for op in service.operations:
        for attribute in ['name', 'formatOptions', 'methods']:
            assert hasattr(op, attribute)
    # Check all contents implement IContentMetadata as a dictionary
    isinstance(service.contents, dict)
    # Check any item (WCS coverage, WMS layer etc) from the contents of each service
    # Check it conforms to IContentMetadata interface
    # get random item from contents dictionary -has to be a nicer way to do this!
    content = service.contents[list(service.contents.keys())[0]]
    for attribute in ['id', 'title', 'boundingBox', 'boundingBoxWGS84', 'crsOptions', 'styles', 'timepositions']:
        assert hasattr(content, attribute)



from tests.utils import scratch_file
from tests.utils import service_ok

from owslib.wmts import WebMapTileService

import pytest

SERVICE_URL = 'http://map1c.vis.earthdata.nasa.gov/wmts-geo/wmts.cgi'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMTS service is unreachable")
@pytest.mark.skip(reason="WMTS service not responding correctly")
def test_wmts():
    # Find out what a WMTS has to offer. Service metadata:
    wmts = WebMapTileService(SERVICE_URL)
    assert wmts.identification.type == 'OGC WMTS'
    assert wmts.identification.version == '1.0.0'
    assert wmts.identification.title == 'NASA Global Imagery Browse Services for EOSDIS'
    bytearray(wmts.identification.abstract, 'utf-8')
    bytearray(b'Near real time imagery from multiple NASA instruments')
    assert wmts.identification.keywords == ['World', 'Global']
    # Service Provider:
    assert wmts.provider.name == 'National Aeronautics and Space Administration'
    assert wmts.provider.url == 'https://earthdata.nasa.gov/'
    # Available Layers:
    assert len(wmts.contents.keys()) > 0
    assert sorted(list(wmts.contents))[0] == 'AIRS_CO_Total_Column_Day'
    # Fetch a tile (using some defaults):
    tile = wmts.gettile(layer='MODIS_Terra_CorrectedReflectance_TrueColor',
                        tilematrixset='EPSG4326_250m', tilematrix='0',
                        row=0, column=0, format="image/jpeg")
    out = open(scratch_file('nasa_modis_terra_truecolour.jpg'), 'wb')
    bytes_written = out.write(tile.read())
    out.close()
    # Test styles for several layers
    # TODO: fix dict order
    # assert wmts.contents['MLS_SO2_147hPa_Night'].styles == {'default': {'isDefault': True, 'title': 'default'}}
    assert wmts.contents['MLS_SO2_147hPa_Night'].styles['default']['isDefault'] is True
    # assert wmts.contents['MLS_SO2_147hPa_Night'].styles == {'default': {'isDefault': True, 'title': 'default'}}
    assert wmts.contents['MLS_SO2_147hPa_Night'].styles['default']['isDefault'] is True


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMTS service is unreachable")
@pytest.mark.skip(reason="WMTS service not responding correctly")
def test_wmts_example_build_tile_request():
    """
    Example for wmts.buildTileRequest
    """
    wmts = WebMapTileService(SERVICE_URL)
    wmts.buildTileRequest(
        layer='VIIRS_CityLights_2012',
        tilematrixset='EPSG4326_500m',
        tilematrix='6',
        row=4, column=4)
    request = 'SERVICE=WMTS&REQUEST=GetTile&VERSION=1.0.0&\
LAYER=VIIRS_CityLights_2012&STYLE=default&TILEMATRIXSET=EPSG4326_500m&\
TILEMATRIX=6&TILEROW=4&TILECOL=4&FORMAT=image%2Fjpeg'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMTS service is unreachable")
@pytest.mark.skip(reason="WMTS service not responding correctly")
def test_wmts_example_get_title():
    """
    Example for wmts.getTitle
    """
    wmts = WebMapTileService(SERVICE_URL)
    img = wmts.gettile(
        layer='VIIRS_CityLights_2012',
        tilematrixset='EPSG4326_500m',
        tilematrix='6',
        row=4, column=4)
    out = open('tile.jpg', 'wb')
    bytes_written = out.write(img.read())
    out.close()


EXAMPLE_SERVICE_URL = "http://tile.informatievlaanderen.be/ws/raadpleegdiensten/wmts"


def test_wmts_example_informatievlaanderen():
    wmts = WebMapTileService(EXAMPLE_SERVICE_URL)
    assert wmts.identification.type == 'OGC:WMTS'
    assert wmts.identification.version == '1.0.0'
    assert wmts.identification.title == 'agentschap Digitaal Vlaanderen WMTS service'
    # assert sorted(list(wmts.contents))[:5] == ['abw', 'ferraris', 'frickx', 'grb_bsk', 'grb_bsk_grijs']


@pytest.mark.online
@pytest.mark.skipif(not service_ok(EXAMPLE_SERVICE_URL),
                    reason="WMTS service is unreachable")
def test_wmts_without_serviceprovider_tag():
    # Test a WMTS without a ServiceProvider tag in Capababilities XML
    from owslib.wmts import WebMapTileService
    wmts = WebMapTileService(SERVICE_URL_ARCGIS)


SERVICE_URL_REST = 'https://www.basemap.at/wmts/1.0.0/WMTSCapabilities.xml'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL_REST),
                    reason="WMTS service is unreachable")
def test_wmts_rest_only():
    # Test a WMTS with REST only
    from owslib.wmts import WebMapTileService
    wmts = WebMapTileService(SERVICE_URL_REST)
    tile = wmts.gettile(layer="bmaporthofoto30cm", tilematrix="10", row=357, column=547)
    assert(tile.info()['Content-Type'] == 'image/jpeg')



#!/usr/bin/python
# -*- coding: UTF-8 -*-
__author__ = 'Juergen Weichand'


from owslib.wfs import WebFeatureService

wfs = WebFeatureService(url='http://geoserv.weichand.de:8080/geoserver/wfs', version='2.0.0', timeout=30)

# List StoredQueries
print('\nStoredQueries for %s' % wfs.identification.title)
for storedquery in wfs.storedqueries:
    print(storedquery.id, storedquery.title)

# List Parameter for a given StoredQuery
storedquery = wfs.storedqueries[5]
print('\nStoredQuery parameters for %s' % storedquery.id)
for parameter in storedquery.parameters:
    print(parameter.name, parameter.type)

# GetFeature StoredQuery
print('\nDownload data from %s' % wfs.identification.title)
response = wfs.getfeature(
    storedQueryID='GemeindeByGemeindeschluesselEpsg31468',
    storedQueryParams={'gemeindeschluessel':'09162000'})
out = open('/tmp/test-storedquery.gml', 'wb')
out.write(response.read())
out.close()
print('... done')



from tests.utils import scratch_file
from tests.utils import service_ok

from owslib.wmts import WebMapTileService

import pytest

SERVICE_URL = "http://geoserv.weichand.de/mapproxy/wmts/1.0.0/WMTSCapabilities.xml"


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WMTS service is unreachable")
def test_wmts_rest_only():
    # ServiceMetadata
    wmts = WebMapTileService(SERVICE_URL)
    assert wmts.identification.type == 'OGC WMTS'
    assert wmts.identification.version == '1.0.0'
    assert wmts.identification.title == 'WMTS-Testserver DOP80'
    # Content
    assert sorted(list(wmts.contents)) == ['dop80']
    # RESTful WMTS
    assert wmts.restonly
    resource = wmts.buildTileResource(
        layer='dop80', tilematrixset='webmercator', tilematrix='11', row='706', column='1089')
    assert resource == 'http://geoserv.weichand.de/mapproxy/wmts/dop80/webmercator/11/1089/706.png'

    tile = wmts.gettile(
        layer='dop80', tilematrixset='webmercator', tilematrix='11', row='706', column='1089')
    out = open(scratch_file('bvv_bayern_dop80.png'), 'wb')
    bytes_written = out.write(tile.read())
    out.close()



# Simulate a WPS Execute invocation with INVALID arguments.
# This test does not execute any live HTTP request, rather it parses XML files containing pre-made HTTP responses.

from tests.utils import resource_file
from owslib.wps import WebProcessingService


def test_wps_execute_invalid_request():
    # Initialize WPS client
    wps = WebProcessingService(
        'http://cida.usgs.gov/gdp/process/WebProcessingService',
        skip_caps=True)

    # Submit fake invocation of Execute operation using cached HTTP request and response
    request = open(resource_file('wps_USGSExecuteInvalidRequest.xml'), 'rb').read()
    response = open(resource_file('wps_USGSExecuteInvalidRequestResponse.xml'), 'rb').read()
    execution = wps.execute(None, [], request=request, response=response)

    assert execution.isComplete() is True

    # Display errors
    ex = execution.errors[0]
    assert ex.code is None
    assert ex.locator is None
    assert ex.text == 'Attribute null not found in feature collection'



# Test generation of a WPS request from input arguments.
# The specific request involves a "reprojectCoords" process submitted to the PML WPS service.

from tests.utils import resource_file, compare_xml
from owslib.wps import WebProcessingService, WPSExecution
from owslib.etree import etree


def test_wps_request5():
    # Process input/output arguments
    processid = "reprojectCoords"
    inputs = [("coords", "http://rsg.pml.ac.uk/wps/testdata/coords.txt"),
              ("outputSRS", "EPSG:32630"),
              ("inputSRS", "EPSG:4326")]

    # build XML request for WPS process execution
    execution = WPSExecution()
    requestElement = execution.buildRequest(processid, inputs)
    request = etree.tostring(requestElement)

    # Compare to cached XML request
    _request = open(resource_file('wps_PMLExecuteRequest5.xml'), 'rb').read()
    assert compare_xml(request, _request) is True



from tests.utils import service_ok
from tests.utils import resource_file

from owslib.wms import WebMapService
import os
from collections import OrderedDict

import pytest


def test_wms_capabilities():
    # Fake a request to a WMS Server using saved doc from
    # http://wms.jpl.nasa.gov/wms.cgi.
    xml = open(resource_file('wms_JPLCapabilities.xml'), 'rb').read()
    wms = WebMapService('url', version='1.1.1', xml=xml)

    # Test capabilities
    # -----------------

    assert wms.identification.type == 'OGC:WMS'
    assert wms.identification.version == '1.1.1'
    assert wms.identification.title == 'JPL Global Imagery Service'
    assert wms.identification.abstract == 'WMS Server maintained by JPL, worldwide satellite imagery.'
    assert wms.identification.keywords == ['ImageryBaseMapsEarthCover', 'Imagery',
                                           'BaseMaps', 'EarthCover', 'JPL', 'Jet Propulsion Laboratory',
                                           'Landsat', 'WMS', 'SLD', 'Global']
    assert wms.identification.accessconstraints == 'Server is load limited'
    assert wms.identification.fees == 'none'
    assert wms.provider.name == 'JPL'
    assert wms.provider.url == 'http://OnEarth.jpl.nasa.gov/index.html'

    # Check contact info (some of it is missing)
    assert wms.provider.contact.name == 'Lucian Plesea'
    assert wms.provider.contact.email == 'lucian.plesea@jpl.nasa.gov'
    wms.provider.contact.address
    wms.provider.contact.city
    wms.provider.contact.country
    wms.provider.contact.region
    wms.provider.contact.postcode
    assert wms.provider.contact.organization == 'JPL'
    wms.provider.contact.position

    # Test available content layers
    assert isinstance(wms.items(), list) is True
    assert isinstance(wms.contents, OrderedDict) is True

    # NOTE: Not sure this dictionary interface is right...??
    assert sorted(wms.contents.keys()) == ['BMNG', 'daily_afternoon', 'daily_planet',
                                           'gdem', 'global_mosaic', 'global_mosaic_base',
                                           'huemapped_srtm', 'modis', 'srtm_mag', 'srtmplus',
                                           'us_colordem', 'us_elevation', 'us_landsat_wgs84',
                                           'us_ned', 'worldwind_dem']

    assert sorted([wms[layer].id for layer in wms.contents]) == ['BMNG', 'daily_afternoon', 'daily_planet',
                                                                 'gdem', 'global_mosaic', 'global_mosaic_base',
                                                                 'huemapped_srtm', 'modis', 'srtm_mag', 'srtmplus',
                                                                 'us_colordem', 'us_elevation', 'us_landsat_wgs84',
                                                                 'us_ned', 'worldwind_dem']
    # Test single item accessor
    assert wms['global_mosaic'].title == 'WMS Global Mosaic, pan sharpened'
    assert wms['global_mosaic'].keywords == []

    ['GlobalMosaic', 'Imagery', 'BaseMaps', 'EarthCover', 'JPL', 'Jet Propulsion Laboratory',
     'Landsat', 'WMS', 'SLD', 'Global']

    wms['global_mosaic'].boundingBox
    assert wms['global_mosaic'].boundingBoxWGS84 == (-180.0, -60.0, 180.0, 84.0)
    assert sorted(wms['global_mosaic'].crsOptions) == ['AUTO:42003', 'EPSG:4326']

    x = wms['global_mosaic'].styles
    assert x == {'pseudo_bright': {'title': 'Pseudo-color image (Uses IR and Visual bands, 542 mapping), gamma 1.5'},
                 'pseudo': {'title': '(default) Pseudo-color image, pan sharpened (Uses IR and Visual bands, 542 mapping), gamma 1.5'},  # noqa
                 'visual': {'title': 'Real-color image, pan sharpened (Uses the visual bands, 321 mapping), gamma 1.5'},
                 'pseudo_low': {'title': 'Pseudo-color image, pan sharpened (Uses IR and Visual bands, 542 mapping)'},
                 'visual_low': {'title': 'Real-color image, pan sharpened (Uses the visual bands, 321 mapping)'},
                 'visual_bright': {'title': 'Real-color image (Uses the visual bands, 321 mapping), gamma 1.5'}}

    # Expecting a KeyError for invalid names
    with pytest.raises(KeyError):
        wms['utterly bogus'].title
        pytest.fail("Expecting a KeyError for invalid names")

    # Test operations
    assert sorted([op.name for op in wms.operations]) == ['GetCapabilities', 'GetMap', 'GetTileService']

    x = wms.getOperationByName('GetMap').methods
    assert x == [{'type': 'Get', 'url': 'http://wms.jpl.nasa.gov/wms.cgi?'}]

    assert wms.getOperationByName('GetMap').formatOptions == ['image/jpeg', 'image/png', 'image/geotiff',
                                                              'image/tiff', 'application/vnd.google-earth.kml+xml']

    # Test exceptions
    assert wms.exceptions == ['application/vnd.ogc.se_xml']


def test_wms_capabilities_style_without_title():
    xml = open(resource_file('wms_geoserver-cap_no_title.xml'), 'rb').read()
    wms = WebMapService('url', version='1.1.1', xml=xml)


SERVICE_URL = 'http://giswebservices.massgis.state.ma.us/geoserver/wms'


@pytest.mark.xfail(reason="online test")
@pytest.mark.online
def test_wms_getmap():
    assert service_ok(SERVICE_URL)
    # Lastly, test the getcapabilities and getmap methods
    wms = WebMapService(SERVICE_URL, version='1.1.1')



from tests.utils import service_ok

import pytest

from owslib.csw import CatalogueServiceWeb as cs

SERVICE_URL = 'http://demo.pycsw.org/cite/csw'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason='service is unreachable')
def test_csw_pycsw_skip_caps():
    c = cs(SERVICE_URL, skip_caps=True)
    c.getrecords2()
    assert c.results.get('returned') > 0
    assert c.results.get('nextrecord') > 0
    assert c.results.get('matches') > 0



from tests.utils import service_ok

import pytest

from owslib.ogcapi.records import Records

SERVICE_URL = 'https://demo.pycsw.org/cite'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason='service is unreachable')
def test_ogcapi_records_pycsw():
    w = Records(SERVICE_URL)

    assert w.url == 'https://demo.pycsw.org/cite/'
    assert w.url_query_string is None

    api = w.api()
    assert api['components']['parameters'] is not None
    paths = api['paths']
    assert paths is not None
    assert paths['/collections/{collectionId}'] is not None

    conformance = w.conformance()
    assert len(conformance['conformsTo']) == 12

    collections = w.collections()
    assert len(collections) > 0

    record_collections = w.records()
    assert record_collections == ['metadata:main']

    pycsw_cite_demo = w.collection('metadata:main')
    assert pycsw_cite_demo['id'] == 'metadata:main'
    assert pycsw_cite_demo['title'] == 'pycsw OGC CITE demo and Reference Implementation'  # noqa
    assert pycsw_cite_demo['itemType'] == 'record'
    assert w.request == 'https://demo.pycsw.org/cite/collections/metadata:main'  # noqa
    assert w.response is not None
    assert isinstance(w.response, dict)

    pycsw_cite_demo_queryables = w.collection_queryables('metadata:main')
    assert len(pycsw_cite_demo_queryables['properties'].keys()) == 12

    # Minimum of limit param is 1
    with pytest.raises(RuntimeError):
        pycsw_cite_demo_query = w.collection_items('metadata:main', limit=0)

    pycsw_cite_demo_query = w.collection_items('metadata:main', limit=1)
    assert pycsw_cite_demo_query['numberMatched'] == 12
    assert pycsw_cite_demo_query['numberReturned'] == 1
    assert len(pycsw_cite_demo_query['features']) == 1

    pycsw_cite_demo_query = w.collection_items('metadata:main', q='lorem')
    assert pycsw_cite_demo_query['numberMatched'] == 5
    assert pycsw_cite_demo_query['numberReturned'] == 5
    assert len(pycsw_cite_demo_query['features']) == 5

    cql_text = "title LIKE 'Lorem%'"
    pycsw_cite_demo_query = w.collection_items('metadata:main', filter=cql_text)
    assert pycsw_cite_demo_query['numberMatched'] == 2
    assert pycsw_cite_demo_query['numberReturned'] == 2
    assert len(pycsw_cite_demo_query['features']) == 2

    cql_json = {'op': '=', 'args': [{'property': 'title'}, 'Lorem ipsum']}
    pycsw_cite_demo_query = w.collection_items('metadata:main', cql=cql_json)
    assert pycsw_cite_demo_query['numberMatched'] == 1
    assert pycsw_cite_demo_query['numberReturned'] == 1
    assert len(pycsw_cite_demo_query['features']) == 1



# -*- coding: UTF-8 -*-
from owslib.sos import SensorObservationService

from tests.utils import service_ok

import pytest

SERVICE_URL = 'http://sos.irceline.be/sos'


@pytest.mark.online
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="SOS service is unreachable")
def test_sos_caps():
    sos = SensorObservationService(SERVICE_URL)
    assert str(sos.contents['81102 - PM10']) == "Offering id: 81102 - PM10, name: Particulate Matter < 10 µm"
    assert repr(sos.contents['81102 - PM10']) == "<SosObservationOffering 'Particulate Matter < 10 µm'>"



from tests.utils import scratch_file
from tests.utils import service_ok
from owslib.wcs import WebCoverageService


import pytest
import datetime

SERVICE_URL = 'http://ows.rasdaman.org/rasdaman/ows'


@pytest.mark.online
@pytest.mark.skip(reason="WCS service is broken (#494)")
@pytest.mark.skipif(not service_ok(SERVICE_URL),
                    reason="WCS service is unreachable")
def test_wcs_200():
    """
    Web Coverage Service
    WCS Version 2.0.x

    rewritten doctest/wcs_200.txt
    """
    wcs = WebCoverageService(SERVICE_URL, version="2.0.1")
    assert wcs.version == '2.0.1'
    assert wcs.url == SERVICE_URL
    assert wcs.identification.title == 'rasdaman'
    assert wcs.identification.service == 'OGC WCS'
    assert wcs.provider.name == 'Jacobs University Bremen'
    assert 'AvgLandTemp' in wcs.contents.keys()
    assert len(wcs.contents.keys()) >= 20
    cvg = wcs.contents['AvgLandTemp']
    assert cvg.boundingboxes[0]['bbox'] == (-90, -180,
                                            90, 180)
    assert cvg.timelimits == [datetime.datetime(2000, 2, 1, 0, 0), datetime.datetime(2015, 6, 1, 0, 0)]
    assert cvg.timepositions[0:5] == [datetime.datetime(2000, 2, 1, 0, 0), datetime.datetime(2000, 3, 1, 0, 0),
                                      datetime.datetime(2000, 4, 1, 0, 0), datetime.datetime(2000, 5, 1, 0, 0),
                                      datetime.datetime(2000, 6, 1, 0, 0)]
    assert cvg.supportedFormats == ['application/gml+xml', 'image/jpeg', 'image/png', 'image/tiff', 'image/bmp',
                                    'image/jp2', 'application/netcdf', 'text/csv', 'application/json',
                                    'application/dem', 'application/x-ogc-dted', 'application/x-ogc-ehdr',
                                    'application/x-ogc-elas', 'application/x-ogc-envi', 'application/x-ogc-ers',
                                    'application/x-ogc-fit', 'application/x-ogc-fits', 'image/gif',
                                    'application/x-netcdf-gmt', 'application/x-ogc-gs7bg', 'application/x-ogc-gsag',
                                    'application/x-ogc-gsbg', 'application/x-ogc-gta', 'application/x-ogc-hf2',
                                    'application/x-erdas-hfa', 'application/x-ogc-ida', 'application/x-ogc-ingr',
                                    'application/x-ogc-isis2', 'application/x-erdas-lan', 'application/x-ogc-mff2',
                                    'application/x-ogc-nitf', 'application/x-ogc-paux', 'application/x-ogc-pcidsk',
                                    'application/x-ogc-pcraster', 'application/x-ogc-pdf', 'application/x-ogc-pnm',
                                    'text/x-r', 'application/x-ogc-rmf', 'image/x-sgi', 'application/x-ogc-vrt',
                                    'image/xpm', 'application/x-ogc-zmap']
    assert cvg.grid.axislabels == ['Lat', 'Long', 'ansi']
    assert cvg.grid.dimension == 3
    assert cvg.grid.lowlimits == ['0', '0', '0']
    assert cvg.grid.highlimits == ['1799', '3599', '184']
    covID = 'AvgLandTemp'
    time_subset = ("ansi", "2000-02-01T00:00:00Z")
    lat_subset = ('Lat', 40, 50)
    long_subset = ('Long', -10, 0)
    formatType = 'application/netcdf'
    output = wcs.getCoverage(identifier=[covID], format=formatType, subsets=[long_subset, lat_subset, time_subset])
    f = open(scratch_file('test_wcs_200.nc'), 'wb')
    bytes_written = f.write(output.read())
    f.close()
